/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   SELOR - Servidor de Emails Livre Orientado a Regras                           
   Programado por Lucas Priori                                                   
                                                                                 
   Linux - make linux                                                            
       or  make linux_nocap                                                      
   FreeBSD - gmake freebsd                                                       
                                                                                 
  *******************************************************************            
  Selor e um agente MTA para troca de emails via protocolo SMTP                 
  Copyright (C) 2012,2013,2014  Lucas Priori                                     
                                                                                 
  This program is free software; you can redistribute it and/or                  
  modify it under the terms of the GNU General Public License                    
  as published by the Free Software Foundation; either version 2                 
  of the License, or (at your option) any later version.                         
                                                                                 
  This program is distributed in the hope that it will be useful,                
  but WITHOUT ANY WARRANTY; without even the implied warranty of                 
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                  
  GNU General Public License for more details.                                   
                                                                                 
  You should have received a copy of the GNU General Public License              
  along with this program; if not, write to the Free Software                    
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

#include<stdio.h>
#include<stdlib.h>
#include<string.h>
#include<fcntl.h>
#include<unistd.h>
#include<errno.h>
#include<sched.h>
#include<time.h>
#include<signal.h>
#include<netinet/in.h>
#include<resolv.h>
#include<netdb.h>
#include<stdarg.h>
#include<arpa/inet.h>
#include<sys/wait.h>
#include<sys/sem.h>
//#include<netinet/in.h>

// ---
#include "inc/global.h"
#include "inc/selor_utils.h"
#include "inc/selor_dns.h"
#include "inc/selor_mod.h"
#include "inc/selor_tls.h"
#include "inc/selor_rules.h"
#include "inc/selor_md5.h"


// ---
char srs_caracs[256]=
{ 'w','x','y','z','0','1','2','3','4','5','6','7','8','9','A','B',
  'A','B','C','D','E','F','G','H','I','J','K','L','M','N','O','P',
  'g','h','i','j','k','l','m','n','o','p','q','r','s','t','u','v',
  'Q','R','S','T','U','V','W','X','Y','Z','a','b','c','d','e','f',
  'S','T','U','V','W','X','Y','Z','a','b','c','d','e','f','g','h',
  'C','D','E','F','G','H','I','J','K','L','M','N','O','P','Q','R',
  'k','l','m','n','o','p','q','r','s','t','u','v','w','x','y','z',
  'y','z','0','1','2','3','4','5','6','7','8','9','A','B','C','D',
  'i','j','k','l','m','n','o','p','q','r','s','t','u','v','w','x',
  'E','F','G','H','I','J','K','L','M','N','O','P','Q','R','S','T',
  'W','X','Y','Z','a','b','c','d','e','f','g','h','i','j','k','l',
  '0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F',
  'U','V','W','X','Y','Z','a','b','c','d','e','f','g','h','i','j',
  'G','H','I','J','K','L','M','N','O','P','Q','R','S','T','U','V',
  '2','3','4','5','6','7','8','9','A','B','C','D','E','F','G','H',
  'm','n','o','p','q','r','s','t','u','v','w','x','y','z','0','1'
};



//======================================================
// Funcao para converter o IP numerico em characteres   
/*char *conv_ip(int ip){
 char buffer_local[20];
 
 sprintf(buffer_local, "%d.%d.%d.%d", ip & 0xFF, ip>>8 & 0xFF, ip>>16 & 0xFF, ip>>24 & 0xFF);
 return buffer_local;
}*/



//======================================================
// Procura um texto dentro de uma string                
// Retornos:                                            
// NULL = texto nao encontro                            
// ou o ponteiro do texto dentro da string              
char *proc_str(char *string, char *texto){
 int  tam_string;
 int  tam_texto;
 int  cont1=0;
 int  tam_a_comparar=0;
 
 tam_string=strlen(string);
 tam_texto=strlen(texto);
 if(!tam_string && !tam_texto)return string;
 if(!tam_texto)return NULL;
 if(tam_string < tam_texto)return NULL;
 tam_a_comparar=tam_string-tam_texto;
 for(cont1=0; cont1<=tam_a_comparar; cont1++)if(!cmp_str(&string[cont1], texto, tam_texto))return &string[cont1];
 return NULL;
}

//======================================================
// Funcao de analise de string de modulos               
void seq_modules(int *seq_ptr, char *mod_string){
 char *ptr_aux1;
 int cont1=0;
 int cont2=0;
 
 if(!mod_string)return;
 while(*mod_string == ' ' || *mod_string == 9)mod_string++;
 ptr_aux1=mod_string;
 while(ptr_aux1 && *ptr_aux1){
  if((ptr_aux1=strchr(mod_string, ' ')))*ptr_aux1++=0;
  // -- Procurando modulos --
  for(cont1=0; cont1<num_mods; cont1++){
   if(!strcmp(mod_names[cont1].mod_id, mod_string)){
    seq_ptr[cont2++]=mod_names[cont1].mod_index;
    seq_ptr[cont2]=0xFF; // Marcando fim de lista de modulos
    break;
   }
  }
  mod_string=ptr_aux1;
 }
}

//======================================================
// Grava um parametro em sua respectiva variavel opcao  
// Retornos:                                            
// 0 = Opcao Invalida                                   
// Ou o nmero da Opcao                                 
int def_parametro(char *opcao, char *parametro){
 void *to_tmp;
 void *mx_map_tmp;
 void *mx_index_tmp;
 char *ptr_aux1;
 char temp1;
 unsigned int DEF_max_rcpts_tmp;
 unsigned int novo_tam_mem_rcpts;
 
 switch(*opcao){
 case 'M':
  if(!strcmp(opcao, nome_op[Module])){
   if(num_mods>=MAX_modules){
    sprintf(buffer_log, "Too many db modules. Max_modules is %d\n", MAX_modules);
    log_msg(DEF_warning_log);
    return 0;
   }
   selor_strncpy(DEF_module[num_mods++], parametro, TAM_defs);
   return Module;
  }else if(!strcmp(opcao, nome_op[Msg_ok])){
   selor_strncpy(DEF_msg_ok, parametro, sizeof(DEF_msg_ok));
   return Msg_ok;
  }else if(!strcmp(opcao, nome_op[Msg_lavg])){
   selor_strncpy(DEF_msg_lavg, parametro, sizeof(DEF_msg_lavg));  
   return Msg_lavg;
  }else if(!strcmp(opcao, nome_op[Max_rcpts])){
   DEF_max_rcpts_tmp=DEF_max_rcpts;
   DEF_max_rcpts=atoi(parametro);
   if(to){
    // Realocando espaco dos rcpts
    novo_tam_mem_rcpts=sizeof(struct dados_email) * DEF_max_rcpts;   // Novo tamanho de memoria
    if(novo_tam_mem_rcpts > tam_mem_rcpts){
     to_tmp=realloc(to, novo_tam_mem_rcpts);
     mx_map_tmp=realloc(mx_map, sizeof(struct str_mx_map) * DEF_max_rcpts);
     mx_index_tmp=realloc(mx_index, sizeof(int) * DEF_max_rcpts);
     if(to_tmp && mx_map_tmp && mx_index_tmp){
      to=to_tmp;
      mx_map=mx_map_tmp;
      mx_index=mx_index_tmp;
      tam_mem_rcpts=novo_tam_mem_rcpts;
      atual_rcpt=to;
     }else {
      // Erro alocando mais espaco. Vamos preservar o antigo e avisar no log de erros
      //snprintf(buffer_log, TAM_buffer_log, "ERROR: Max_rcpts %ld -> realloc memory error\n", DEF_max_rcpts);
      selor_superstrncpy(buffer_log, TAM_buffer_log, "ERROR: Max_rcpts ", itoa(DEF_max_rcpts, 0), " -> realloc memory error\n", NULL);
      log_msg(DEF_warning_log);
      DEF_max_rcpts=DEF_max_rcpts_tmp;
      if(to_tmp)to=to_tmp;
      if(mx_map_tmp)mx_map=mx_map_tmp;
      if(mx_index_tmp)mx_index=mx_index_tmp;
     }
    }
   }
   return Max_rcpts;
  }else if(!strcmp(opcao, nome_op[Max_qprocs])){
   DEF_max_qprocs=atoi(parametro);
   return Max_qprocs;
  }else if(!strcmp(opcao, nome_op[Modseq_auth])){
   selor_strncpy(DEF_modseq_auth, parametro, sizeof(DEF_modseq_auth));  
   seq_modules(modseq_auth, parametro);
   return Modseq_auth;
  }else if(!strcmp(opcao, nome_op[Modseq_domain])){
   selor_strncpy(DEF_modseq_domain, parametro, sizeof(DEF_modseq_domain));
   seq_modules(modseq_domain, parametro);
   return Modseq_domain;
  }else if(!strcmp(opcao, nome_op[Modseq_mail])){
   selor_strncpy(DEF_modseq_mail, parametro, sizeof(DEF_modseq_mail));
   seq_modules(modseq_mail, parametro);
   return Modseq_mail;
  }else if(!strcmp(opcao, nome_op[Modseq_alias])){
   selor_strncpy(DEF_modseq_alias, parametro, sizeof(DEF_modseq_alias));
   seq_modules(modseq_alias, parametro);
   return Modseq_alias;
  }else if(!strcmp(opcao, nome_op[Mx_alias_ip])){
   DEF_mx_alias_ip=atoi(parametro);
   return Mx_alias_ip;
  }else if(!strcmp(opcao, nome_op[Msg_goodbye])){
   selor_strncpy(DEF_msg_goodbye, parametro, sizeof(DEF_msg_goodbye));  
   return Msg_goodbye;
  }else if(!strcmp(opcao, nome_op[Msg_auth_ok])){
   selor_strncpy(DEF_msg_auth_ok, parametro, sizeof(DEF_msg_auth_ok));
   return Msg_auth_ok;
  }else if(!strcmp(opcao, nome_op[Msg_refused])){
   selor_strncpy(DEF_msg_refused, parametro, sizeof(DEF_msg_refused));  
   return Msg_refused;
  }else if(!strcmp(opcao, nome_op[Max_attempts])){
   DEF_max_attempts=atoi(parametro);
   return Max_attempts;
  }else if(!strcmp(opcao, nome_op[Max_mail_size])){
   DEF_max_mail_size=atoi(parametro);
   return Max_mail_size;
  }else if(!strcmp(opcao, nome_op[Max_msgs_per_cnx])){
   DEF_max_msgs_per_cnx=atoi(parametro);
   return Max_msgs_per_cnx;
  }else if(!strcmp(opcao, nome_op[Msg_send_body])){
   selor_strncpy(DEF_msg_send_body, parametro, sizeof(DEF_msg_send_body)); 
   return Msg_send_body;
  }else if(!strcmp(opcao, nome_op[Msg_perm_deny])){
   selor_strncpy(DEF_msg_perm_deny, parametro, sizeof(DEF_msg_perm_deny)); 
   return Msg_perm_deny;
  }else if(!strcmp(opcao, nome_op[Msg_no_mailbox])){
   selor_strncpy(DEF_msg_no_mailbox, parametro, sizeof(DEF_msg_no_mailbox));  
   return Msg_no_mailbox;
  }else if(!strcmp(opcao, nome_op[Max_header_size])){
   DEF_max_header_size=atoi(parametro);
   return Max_header_size;
  }else if(!strcmp(opcao, nome_op[Msg_auth_falied])){
   selor_strncpy(DEF_msg_auth_falied, parametro, sizeof(DEF_msg_auth_falied));
   return Msg_auth_falied;
  }else if(!strcmp(opcao, nome_op[Welcome_msg])){
   selor_strncpy(DEF_welcome_msg, parametro, sizeof(DEF_welcome_msg));
   return Welcome_msg;
  }else if(!strcmp(opcao, nome_op[Msg_mail_toobig])){
   selor_strncpy(DEF_msg_mail_toobig, parametro, sizeof(DEF_msg_mail_toobig));
   return Msg_mail_toobig;
  }else if(!strcmp(opcao, nome_op[Msg_invalid_cmd])){
   selor_strncpy(DEF_msg_invalid_cmd, parametro, sizeof(DEF_msg_invalid_cmd));
   return Msg_invalid_cmd;
  }else if(!strcmp(opcao, nome_op[Max_error_report])){
   DEF_max_error_report=atoi(parametro);
   return Max_error_report;
  }else if(!strcmp(opcao, nome_op[Msg_nodisk_space])){
   selor_strncpy(DEF_msg_nodisk_space, parametro, sizeof(DEF_msg_nodisk_space)); 
   return Msg_nodisk_space;
  }else if(!strcmp(opcao, nome_op[Msg_allowed_rcpt])){
   selor_strncpy(DEF_msg_allowed_rcpt, parametro, sizeof(DEF_msg_allowed_rcpt)); 
   return Msg_allowed_rcpt;
  }else if(!strcmp(opcao, nome_op[Msg_relay_denied])){
   selor_strncpy(DEF_msg_relay_denied, parametro, sizeof(DEF_msg_relay_denied)); 
   return Msg_relay_denied;
  }else if(!strcmp(opcao, nome_op[Msg_full_mailbox])){
   selor_strncpy(DEF_msg_full_mailbox, parametro, sizeof(DEF_msg_full_mailbox)); 
   return Msg_full_mailbox;
  }else if(!strcmp(opcao, nome_op[Msg_toomany_rcpts])){
   selor_strncpy(DEF_msg_toomany_rcpts, parametro, sizeof(DEF_msg_toomany_rcpts));  
   return Msg_toomany_rcpts;
  }else if(!strcmp(opcao, nome_op[Msg_invalid_param])){
   selor_strncpy(DEF_msg_invalid_param, parametro, sizeof(DEF_msg_invalid_param));  
   return Msg_invalid_param;
  }else if(!strcmp(opcao, nome_op[Msg_mail_accepted])){
   selor_strncpy(DEF_msg_mail_accepted, parametro, sizeof(DEF_msg_mail_accepted));  
   return Msg_mail_accepted;
  }else if(!strcmp(opcao, nome_op[Msg_auth_required])){
   selor_strncpy(DEF_msg_auth_required, parametro, sizeof(DEF_msg_auth_required));  
   return Msg_auth_required;
  }else if(!strcmp(opcao, nome_op[Msg_header_toobig])){
   selor_strncpy(DEF_msg_header_toobig, parametro, sizeof(DEF_msg_header_toobig));  
   return Msg_header_toobig;
  }else if(!strcmp(opcao, nome_op[Msg_cannot_resolve])){
   selor_strncpy(DEF_msg_cannot_resolve, parametro, sizeof(DEF_msg_cannot_resolve));
   return Msg_cannot_resolve;
  }else if(!strcmp(opcao, nome_op[Msg_cannot_connect])){
   selor_strncpy(DEF_msg_cannot_connect, parametro, sizeof(DEF_msg_cannot_connect));
   return Msg_cannot_connect;
  }else if(!strcmp(opcao, nome_op[Msg_invalid_domain])){
   selor_strncpy(DEF_msg_invalid_domain, parametro, sizeof(DEF_msg_invalid_domain));
   return Msg_invalid_domain;
  }else if(!strcmp(opcao, nome_op[Msg_allowed_sender])){
   selor_strncpy(DEF_msg_allowed_sender, parametro, sizeof(DEF_msg_allowed_sender));
   return Msg_allowed_sender;
  }else if(!strcmp(opcao, nome_op[Msg_internal_error])){
   selor_strncpy(DEF_msg_internal_error, parametro, sizeof(DEF_msg_internal_error));
   return Msg_internal_error;
  }else if(!strcmp(opcao, nome_op[Msg_sequence_error])){
   selor_strncpy(DEF_msg_sequence_error, parametro, sizeof(DEF_msg_sequence_error));
   return Msg_sequence_error;
  }else if(!strcmp(opcao, nome_op[Msg_transaction_falied])){
   selor_strncpy(DEF_msg_transaction_falied, parametro, sizeof(DEF_msg_transaction_falied)); 
   return Msg_transaction_falied;
  }else if(!strcmp(opcao, nome_op[Msg_mailbox_unavaibble])){
   selor_strncpy(DEF_msg_mailbox_unavaibble, parametro, sizeof(DEF_msg_mailbox_unavaibble)); 
   return Msg_mailbox_unavaibble;
  }else return -1; 
  break;
 case 'S':
  if(!strcmp(opcao, nome_op[Sid])){
   selor_strncpy(DEF_sid, parametro, sizeof(DEF_sid));
   return Sid;
  }else if(!strcmp(opcao, nome_op[Srs_key])){
   selor_strncpy(DEF_srs_key, parametro, sizeof(DEF_srs_key));
   return Srs_key;
  }else if(!strcmp(opcao, nome_op[Srs_shift])){
   DEF_srs_shift=atoi(parametro);
   return Srs_shift;
  }else if(!strcmp(opcao, nome_op[Sumary_log])){
   selor_strncpy(DEF_sumary_log, parametro, sizeof(DEF_sumary_log));
   return Sumary_log;
  }else if(!strcmp(opcao, nome_op[Server_name])){
   selor_strncpy(DEF_server_name, parametro, sizeof(DEF_server_name));
   return Server_name;
  }else if(!strcmp(opcao, nome_op[Srs_alias_redir])){
   DEF_srs_alias_redir=atoi(parametro);
   return Srs_alias_redir;
  }else if(!strcmp(opcao, nome_op[Subject_error_msg])){
   selor_strncpy(DEF_subject_error_msg, parametro, sizeof(DEF_subject_error_msg));
   return Subject_error_msg;
  }else if(!strcmp(opcao, nome_op[Seek_df_alias])){
   DEF_seek_df_alias=atoi(parametro);
   return Seek_df_alias;
  }else return -1; 
  break;
 case 'C':
  if(!strcmp(opcao, nome_op[Cp_errors])){
   DEF_cp_errors=atoi(parametro);
   return Cp_errors;
  }else if(!strcmp(opcao, nome_op[Central_log])){
   selor_strncpy(DEF_central_log, parametro, sizeof(DEF_central_log));  
   return Central_log;
  }else if(!strcmp(opcao, nome_op[Cp_errors_path])){
   selor_strncpy(DEF_cp_errors_path, parametro, sizeof(DEF_cp_errors_path));  
   return Cp_errors_path;
  }else if(!strcmp(opcao, nome_op[Connect_timeout])){
   DEF_connect_timeout=atoi(parametro);
   return Connect_timeout;
  }else if(!strcmp(opcao, "Connection_rules")){
   selor_strncpy(DEF_event_connection, parametro, sizeof(DEF_event_connection));
   return Event_connection;
  }else return -1;
  break;
 case 'E':
  if(!strcmp(opcao, nome_op[Event_helo])){
   selor_strncpy(DEF_event_helo, parametro, sizeof(DEF_event_helo));
   return Event_helo;
  }else if(!strcmp(opcao, nome_op[Event_from])){
   selor_strncpy(DEF_event_from, parametro, sizeof(DEF_event_from));
   return Event_from;
  }else if(!strcmp(opcao, nome_op[Event_rcpt])){
   selor_strncpy(DEF_event_rcpt, parametro, sizeof(DEF_event_rcpt));
   return Event_rcpt;
  }else if(!strcmp(opcao, nome_op[Event_data])){
   selor_strncpy(DEF_event_data, parametro, sizeof(DEF_event_data));
   return Event_data;
  }else if(!strcmp(opcao, nome_op[Event_limit])){
   selor_strncpy(DEF_event_limit, parametro, sizeof(DEF_event_limit));
   return Event_limit;
  }else if(!strcmp(opcao, nome_op[Event_connection])){
   selor_strncpy(DEF_event_connection, parametro, sizeof(DEF_event_connection));
   return Event_connection;
  }else if(!strcmp(opcao, nome_op[Event_error_report])){
   selor_strncpy(DEF_event_error_report, parametro, sizeof(DEF_event_error_report));
   return Event_error_report;
  }else if(!strcmp(opcao, nome_op[Event_qend_process])){
   selor_strncpy(DEF_event_qend_process, parametro, sizeof(DEF_event_qend_process));
   return Event_qend_process;
  }else if(!strcmp(opcao, nome_op[Event_dl_rcpt])){
   selor_strncpy(DEF_event_dl_rcpt, parametro, sizeof(DEF_event_dl_rcpt));
   return Event_dl_rcpt;
  }else if(!strcmp(opcao, nome_op[Event_lrcpt_refused])){
   selor_strncpy(DEF_event_lrcpt_refused, parametro, sizeof(DEF_event_lrcpt_refused));
   return Event_lrcpt_refused;
  }else return -1;
  break;
 case 'D':
  if(!strcmp(opcao, nome_op[Dlocal])){
   selor_strncpy(DEF_dlocal, parametro, sizeof(DEF_dlocal));
   return Dlocal;
  }else if(!strcmp(opcao, "Data_rules")){
   selor_strncpy(DEF_event_data, parametro, sizeof(DEF_event_data));
   return Event_data;
  }else if(!strcmp(opcao, nome_op[Df_domain])){
   selor_strncpy(DEF_df_domain, parametro, sizeof(DEF_df_domain));
   return Df_domain;
  }else if(!strcmp(opcao, nome_op[Dlocal_max_procs])){
   DEF_dlocal_max_procs=atoi(parametro);
   return Dlocal_max_procs;
  }else if(!strcmp(opcao, nome_op[Dlocal_type])){
   DEF_dlocal_type=atoi(parametro);
   return Dlocal_type;
  }else return -1; 
  break;
 case 'R':
  if(!strcmp(opcao, "Rcpt_rules")){
   selor_strncpy(DEF_event_rcpt, parametro, sizeof(DEF_event_rcpt));
   return Event_rcpt;
  }else if(!strcmp(opcao, nome_op[Resolve_all])){
   DEF_resolve_all=atoi(parametro);
   return Resolve_all;
  }else if(!strcmp(opcao, nome_op[Report_from])){
   selor_strncpy(DEF_report_from, parametro, sizeof(DEF_report_from));
   return Report_from;
  }else if(!strcmp(opcao, nome_op[Report_file])){
   selor_strncpy(DEF_report_file, parametro, sizeof(DEF_report_file));
   return Report_file;
  }else if(!strcmp(opcao, nome_op[Rcpt_level_check])){
   DEF_rcpt_level_check=atoi(parametro);
   return Rcpt_level_check;
  }else if(!strcmp(opcao, nome_op[Refused_log])){
   selor_strncpy(DEF_refused_log, parametro, sizeof(DEF_refused_log));  
   return Refused_log;
  }else return -1; 
  break;
 case 'L':
  if(!strcmp(opcao, nome_op[Limit_buf])){
   DEF_limit_buf=atoi(parametro);
   if(DEF_limit_buf && DEF_limit_buf < 50)DEF_limit_buf=50;
   return Limit_buf;
  }else if(!strcmp(opcao, nome_op[Limit_cnx])){
   DEF_limit_cnx=atoi(parametro);
   return Limit_cnx;
  }else if(!strcmp(opcao, nome_op[Limit_time])){
   DEF_limit_time=atoi(parametro);
   return Limit_time;
  }else if(!strcmp(opcao, nome_op[Lavg_queue])){
   DEF_lavg_queue=atoi(parametro);
   return Lavg_queue;
  }else if(!strcmp(opcao, "Limit_rules")){
   selor_strncpy(DEF_event_limit, parametro, sizeof(DEF_event_limit));
   return Event_limit;
  }else if(!strcmp(opcao, nome_op[Lavg_negative])){
   DEF_lavg_negative=atoi(parametro);
   return Lavg_negative;
  }else return -1; 
  break;
 case 'P':
  if(!strcmp(opcao, nome_op[Port])){
   DEF_port=atoi(parametro);
   return Port;
  }else if(!strcmp(opcao, nome_op[Pid_file])){
   selor_strncpy(DEF_pid_file, parametro, sizeof(DEF_pid_file));
   return Pid_file;
  }else if(!strcmp(opcao, nome_op[Postmaster])){
   selor_strncpy(DEF_postmaster, parametro, sizeof(DEF_postmaster));
   return Postmaster;
  }else if(!strcmp(opcao, nome_op[Proto_level_check])){
   DEF_proto_level_check=atoi(parametro);
   return Proto_level_check;
  }else return -1; 
  break;
 case 'F': 
  if(!strcmp(opcao, "From_rules")){
   selor_strncpy(DEF_event_from, parametro, sizeof(DEF_event_from));
   return Event_from;
  }else if(!strcmp(opcao, nome_op[From_level_check])){
   DEF_from_level_check=atoi(parametro);
   return From_level_check;
  }else if(!strcmp(opcao, nome_op[Force_alias_seek])){
   DEF_force_alias_seek=atoi(parametro);
   return Force_alias_seek;
  }else return -1; 
  break;
 case 'Q':
  if(!strcmp(opcao, nome_op[Queue_time])){
   DEF_queue_time=atoi(parametro);
   return Queue_time;
  }else if(!strcmp(opcao, nome_op[Queue_path])){
   selor_strncpy(DEF_queue_path, parametro, sizeof(DEF_queue_path));
   return Queue_path;
  }else if(!strcmp(opcao, nome_op[Qsleep_time])){
   DEF_qsleep_time=atoi(parametro);
   if(DEF_qsleep_time > 60)DEF_qsleep_time=60;
   return Qsleep_time;
  }else if(!strcmp(opcao, nome_op[Qlimit_time])){
   DEF_qlimit_time=atoi(parametro);
   return Qlimit_time;
  }else if(!strcmp(opcao, nome_op[Qlimit_max_procs])){
   DEF_qlimit_max_procs=atoi(parametro);
   return Qlimit_max_procs;
  }else if(!strcmp(opcao, "Qend_attempt_rules")){
   selor_strncpy(DEF_event_qend_process, parametro, sizeof(DEF_event_qend_process));
   return Event_qend_process;
  }else return -1; 
  break;
 case 'T':
  if(!strcmp(opcao, nome_op[Timeout])){
   DEF_timeout=atoi(parametro);
   return Timeout;
  }else if(!strcmp(opcao, nome_op[Timezone])){
   selor_strncpy(DEF_timezone, parametro, sizeof(DEF_timezone));
   return Timezone;
  }else if(!strcmp(opcao, nome_op[Temp_path])){
   selor_strncpy(DEF_temp_path, parametro, sizeof(DEF_temp_path));
   return Temp_path;
  }else if(!strcmp(opcao, nome_op[Tarpit_limit])){
   DEF_tarpit_limit=atoi(parametro);
   if(!DEF_tarpit_limit)DEF_tarpit_limit=0xFFFFFFFF;
   return Tarpit_limit;
  }else if(!strcmp(opcao, nome_op[Tarpit_sleep])){
   DEF_tarpit_sleep=atoi(parametro);
   return Tarpit_sleep;
  }else if(!strcmp(opcao, nome_op[Tls_module])){
   selor_strncpy(DEF_tls_module, parametro, sizeof(DEF_tls_module));
   // Procurando por parametro do modulo
   if(!(ptr_aux1=strchr(DEF_tls_module,  ' ')))ptr_aux1=strchr(DEF_tls_module, 9);
   if(ptr_aux1){
    // Parametro existente
    temp1=*ptr_aux1;
    *ptr_aux1++=0;
    ptr_aux1=pular_carac(ptr_aux1, temp1);
    DEF_tls_module_p=ptr_aux1;
   }else DEF_tls_module_p=NULL;
   return Tls_module;
  }else if(!strcmp(opcao, nome_op[Tls_enable])){
   DEF_tls_enable=atoi(parametro);
   return Tls_enable;
  }else if(!strcmp(opcao, nome_op[Tls_delivery])){
   DEF_tls_delivery=atoi(parametro);
   return Tls_delivery;
  }else if(!strcmp(opcao, nome_op[Tls_cafile])){
   selor_strncpy(DEF_tls_cafile, parametro, sizeof(DEF_tls_cafile));
   return Tls_cafile;
  }else if(!strcmp(opcao, nome_op[Tls_pubkey])){
   selor_strncpy(DEF_tls_pubkey, parametro, sizeof(DEF_tls_pubkey));
   return Tls_pubkey;
  }else if(!strcmp(opcao, nome_op[Tls_privkey])){
   selor_strncpy(DEF_tls_privkey, parametro, sizeof(DEF_tls_privkey));
   return Tls_privkey;
  }else return -1; 
  break;
 case 'W': 
  if(!strcmp(opcao, nome_op[Warning_log])){
   selor_strncpy(DEF_warning_log, parametro, sizeof(DEF_warning_log));
   return Warning_log;
  }else return -1; 
  break;
 case 'U': 
  if(!strcmp(opcao, nome_op[User])){
   selor_strncpy(DEF_user, parametro, sizeof(DEF_user));
   return User;
  }else return -1; 
  break;
 case 'G': 
  if(!strcmp(opcao, nome_op[Group])){
   selor_strncpy(DEF_group, parametro, sizeof(DEF_group));
   return Group;
  }else return -1; 
  break;
 case 'A':
  if(!strcmp(opcao, nome_op[Auth_relay])){
   DEF_auth_relay=atoi(parametro);
   return Auth_relay;
  }else if(!strcmp(opcao, nome_op[Auth_tolerance])){
   DEF_auth_tolerance=atoi(parametro);
   return Auth_tolerance;
  }else if(!strcmp(opcao, nome_op[Adlocal_allowed])){
   selor_superstrncpy(DEF_adlocal_allowed, sizeof(DEF_adlocal_allowed), parametro, " ", NULL);   // Acrescento um espaco no final para falicitar na busca dentro da funcao dlocal
   return Adlocal_allowed;
  }else if(!strcmp(opcao, nome_op[Allow_broadcasts])){
   DEF_allow_broadcasts=atoi(parametro);
   return Adlocal_allowed;
  }else return -1;
  break;
 case 'I':
  if(!strcmp(opcao, nome_op[Immed_delivery])){
   DEF_immed_delivery_bk=DEF_immed_delivery=atoi(parametro);
   return Immed_delivery;
  }else if(!strcmp(opcao, nome_op[Incorrect_pass_delay])){
   DEF_incorrect_pass_delay=atoi(parametro);
   return Incorrect_pass_delay;
  }else return -1;
  break;
 case 'H':
  if(!strcmp(opcao, nome_op[Hshow_user])){
   DEF_hshow_user=atoi(parametro);
   return Hshow_user;
  }else return -1;
  break;
 case 'N':  
  if(!strcmp(opcao, nome_op[No_mailbox_delay])){
   DEF_no_mailbox_delay=atoi(parametro);
   return No_mailbox_delay;
  }else return -1;
  break;
 default:
  return -1;
 }
 return -1;
}




//======================================================
// Funcao para bloquear a conexao que expirou o tempo   
void tempoesgotado(int sinal){
 
 if(!wait_log)close(socket_remoto);
 else close(arq_log);
}

//======================================================
// Funcao que retorna o tempo do log em 
char *log_rettempo(void){
 
 time(&tempo);
 timeinfo=localtime(&tempo);
 strcpy(&buffer_log_completo[1], itoa(timeinfo->tm_year+1900, 4));
 strcpy(&buffer_log_completo[6], itoa(timeinfo->tm_mon+1, 2));
 strcpy(&buffer_log_completo[9], itoa(timeinfo->tm_mday, 2));
 strcpy(&buffer_log_completo[12], itoa(timeinfo->tm_hour, 2));
 strcpy(&buffer_log_completo[15], itoa(timeinfo->tm_min, 2));
 strcpy(&buffer_log_completo[18], itoa(timeinfo->tm_sec, 2));
 buffer_log_completo[0]='[';
 buffer_log_completo[5]=buffer_log_completo[8]='-';
 buffer_log_completo[11]=' ';
 buffer_log_completo[14]=buffer_log_completo[17]=':';
 buffer_log_completo[20]=']';
 buffer_log_completo[21]=0;
 
 return log_strtempo;
}


//======================================================
// Funcao para retornar a data no form. correto p emails
void rfc_date(char *retorno, time_t timestamp){
 static char mes[12][4]={"Jan\0", "Feb\0", "Mar\0", "Apr\0", "May\0", "Jun\0", "Jul\0", "Aug\0", "Sep\0", "Oct\0", "Nov\0", "Dec\0"};
 static char dia_semana[7][4]={"Sun\0", "Mon\0", "Tue\0", "Wed\0", "Thu\0", "Fri\0", "Sat\0"};
 
 timeinfo=localtime(&tempo);
 strcpy(retorno, dia_semana[timeinfo->tm_wday]);
 strcpy(&retorno[5], itoa(timeinfo->tm_mday, 2));
 strcpy(&retorno[8], mes[timeinfo->tm_mon]);
 strcpy(&retorno[12], itoa(timeinfo->tm_year+1900, 4));
 strcpy(&retorno[17], itoa(timeinfo->tm_hour, 2));
 strcpy(&retorno[20], itoa(timeinfo->tm_min, 2));
 strcpy(&retorno[23], itoa(timeinfo->tm_sec, 2));
 strcpy(&retorno[26], DEF_timezone);
 retorno[3]=',';
 retorno[4]=' ';
 retorno[7]=' ';
 retorno[11]=' ';
 retorno[16]=' ';
 retorno[19]=':';
 retorno[22]=':';
 retorno[25]=' ';      // Acrescentando espaco para 
}

//======================================================
// Funcao de LOG do servidor                            
// Retornos                                             
// 0 = Sucesso                                          
// 1 = Erro ao abrir o arquivo                          
char log_msg(char *CAM_log){
 char log_tmp[TAM_buffer + 1];
 unsigned int tam_log;
 
 if(!*CAM_log)return 0;   // --> Log desativado
 /*if(*CAM_log == '|'){
  // Log via PIPE
  
 }*/
 if((arq_log=open(CAM_log, O_CREAT | O_WRONLY, 0666)) < 0)return 1;
 
 log_rettempo();
 *buffer_log_juncao=' ';
 tam_log=strlen(buffer_log_completo);
 
 wait_log=1;
 alarm(1);
 lockf(arq_log, F_LOCK, 0);
 alarm(0);
 wait_log=0;

 if(lockf(arq_log, F_TEST, 0) != 0){
  if(errno != EBADF)return 1;
  // Tempo para travar arquivo de Log expirado.
  // Vamos criar um outro                      
  close(arq_log);
  selor_superstrncpy(log_tmp, TAM_buffer, CAM_log, ".", fpid, NULL);
  if((arq_log=open(log_tmp, O_CREAT | O_WRONLY, 0666)) < 0)return 1;
 }
 lseek(arq_log, 0, 2);
 write(arq_log, buffer_log_completo, tam_log);
 lockf(arq_log, F_ULOCK, 0);
 close(arq_log);
 return 0;
}

//======================================================
// Captura alguns sinais                                
// OBS: Esta f. nao precisa ser rapida e foi otimizada p
// ocupar pouco espaco                                  
void capta_sinal(int sinal){
 char nom_sinais[9][10]={"SIGTERM\0", "SIGSEGV\0", "SIGTHUP\0", "SIGTINT\0", "SIGTQUIT\0", "SIGILL\0", "SIGABRT\0", "SIGFPE\0", "SIGTRAP\0"};
 char *ptr_aux1;
 
 // Excluindo arquivo de fila
 ptr_aux1=nom_sinais[0];
 if(sinal == SIGSEGV)ptr_aux1+=10;
 else if(sinal == SIGHUP)ptr_aux1+=20;
 else if(sinal == SIGINT)ptr_aux1+=30;
 else if(sinal == SIGQUIT)ptr_aux1+=40;
 else if(sinal == SIGILL)ptr_aux1+=50;
 else if(sinal == SIGABRT)ptr_aux1+=60;
 else if(sinal == SIGFPE)ptr_aux1+=70;
 else if(sinal == SIGTRAP)ptr_aux1+=80;
 
 if(rem_arq_signal){
  // Arquivo de fila precisa ser removido
  if(cam_filac[0])remove(cam_filac);
  if(cam_filal[0])remove(cam_filal);
 }
 
 sprintf(buffer_log, "(PID %s - %.2f) <-- %s\n", fpid, etapa_global, ptr_aux1);
 log_msg(DEF_warning_log);
 if(to)free(to);
 exit(0);
}

//======================================================
// Computa o vetor de slots livres para controle de pids
// Retorna o numero de slots vazios                     
unsigned int compute_slots(int *processos, unsigned int max_procs){
 unsigned int cont1=0;
 unsigned int num_livres=0;
 int ret_pid;
 
 for(cont1=0; cont1<max_procs; cont1++){
  if(!processos[cont1] || ((waitpid(processos[cont1], &ret_pid, WNOHANG)) < 0)){
   processos[cont1]=0;
   num_livres++;
  }
 }
 return num_livres;
}
//======================================================
// Registra um pid no vetor de slots de pids            
void reg_pid(int *processos, unsigned int max_procs, int num_pid){
 unsigned int cont1=0;
 
 for(cont1=0; cont1<max_procs; cont1++)if(!processos[cont1]){
  processos[cont1]=num_pid;
  return;
 }
}


//======================================================
// Envia um buffer via SOCKET                           
// Retorna o numero de bytes enviados ou -1 em caso de  
// erro                                                 
int selor_send(int num_socket, char *buffer_local, int num_bytes, unsigned int flags){
 unsigned int enviados=0;
 int b_enviados=0;
 
 enviados=num_bytes;
 do{
  if(!tls_ativo)b_enviados=send(num_socket, buffer_local, num_bytes, flags);
  else b_enviados=tls_send(buffer_local, num_bytes);
  if(b_enviados < 0)return -1;
  buffer_local+=b_enviados;
  num_bytes-=b_enviados;
 }while(num_bytes && b_enviados);
 return enviados-num_bytes;
}

//======================================================
// Funcao para recebimento de dados via socket com prot.
// de timeout                                           
int selor_recv(int num_socket, char *buffer_local, int num_bytes, unsigned int flags){
 int recebidos;
 
 alarm(DEF_timeout);
 if(!tls_ativo)recebidos=recv(num_socket, buffer_local, num_bytes, flags);
 else recebidos=tls_recv(buffer_local, num_bytes);
 alarm(0);
 if(recebidos >= 0)buffer_local[recebidos]=0;
 
 return recebidos;
}

//======================================================
// Funo para receber um comando                       
int selor_cmd_recv(int num_socket, char *buffer_local, int num_bytes, unsigned int flags){
 int b_recebidos=0;
 unsigned int tam_total=0;;
 unsigned int resto=num_bytes;
 
 do{
  if((b_recebidos=selor_recv(num_socket, &buffer_local[tam_total], resto, flags)) < 1)return b_recebidos;
  tam_total+=b_recebidos;
  resto-=b_recebidos;
  if(resto < 1)return tam_total;      // Seguranca contra buffer overflow
 }while(tam_total == 1 || buffer_local[tam_total-2] !=13 || buffer_local[tam_total-1] !=10);
 return tam_total;
}


//======================================================
// Recebe uma resposta de protocolo smtp em pacotes     
// separados                                            
// Retorna o numero de bytes em buffer_local            
int selor_smtp_recv(int num_socket, char *buffer_local, int num_bytes, unsigned int flags){
 int b_recebidos;
 unsigned int tam_total;
 char *ptr_aux1;
 
 tam_total=0;
 while(1){
  if((b_recebidos=selor_recv(num_socket, &buffer_local[tam_total], num_bytes, flags)) < 1)return b_recebidos;
  num_bytes-=b_recebidos;
  tam_total+=b_recebidos;
  if(num_bytes < 1)return tam_total;            // Segurana contra buffer overflow
  if(tam_total > 1)ptr_aux1=&buffer_local[tam_total-2];
  else ptr_aux1=buffer_local;
  while(*ptr_aux1 != 10 && ptr_aux1 >= buffer_local)ptr_aux1--;
  ptr_aux1++;
  if(ptr_aux1[3]==' ' && buffer_local[tam_total-1] == 10)return tam_total;
 }
 return tam_total;     // Nunca sera chamado. Mas esta aqui para o compilador nao reclamar
}

//======================================================
// Envia uma resposta SMTP ao host conectado            
void smtp_resp(char *acrescimo, char tipo, char parametro){
 char buffer_local[TAM_buffer + 1];
 char *ptr_aux1=buffer_local;
 char nvl_log=0;     //1 = Recebido | 2 = Enviado | 3 = Enviado e Recebido
 unsigned int tam_buffer=0;
 unsigned int posi_buffer=0;
 
 if(!R_REPLY[0]){
  if(num_smtp_resp == MSG_resp_ehlo){
   // A resposta ao ehlo precisa ser montada aqui
   // pois ela pode ser alterada por alguma regra
   posi_buffer=selor_superstrncpy(buffer_local, TAM_buffer, "250-", hostname, "\r\n", NULL);
   if(DEF_max_mail_size)selor_superstrncpy(&buffer_local[posi_buffer], TAM_buffer-posi_buffer, "250-SIZE ", itoa(DEF_max_mail_size, 0), "\r\n", NULL);
   if(DEF_auth_relay)selor_strncat(buffer_local, "250-AUTH PLAIN LOGIN\r\n", TAM_buffer-1);
   if(DEF_tls_enable && !tls_ativo)selor_strncat(buffer_local, "250-STARTTLS\r\n", TAM_buffer-1);
   tam_buffer=selor_strncat(buffer_local, "250 8BITMIME\r\n", TAM_buffer-1);
  }else {
   if(acrescimo)tam_buffer=selor_superstrncpy(buffer_local, TAM_buffer, smtp_resps[num_smtp_resp], " ", acrescimo, "\r\n", NULL);
   else tam_buffer=selor_superstrncpy(buffer_local, TAM_buffer-2, smtp_resps[num_smtp_resp], "\r\n", NULL);
  }
  selor_send(socket_remoto, buffer_local, tam_buffer, 0);
 }else ptr_aux1=R_REPLY;
 if(!tipo){
  nvl_log=NVL_log_rec;
  if((*ptr_aux1 != '2' && *ptr_aux1 != '3') || num_smtp_resp == MSG_mail_accepted)nvl_log|=NVL_log_env;
  nvl_log&=parametro;
  // Fazendo LOG do comando enviado pelo HOST
  if(nvl_log){
   if(nvl_log == NVL_log_rec)selor_superstrncpy(buffer_log, TAM_buffer_log, "(PID ", fpid, ") <- ", host.ip, " ", smtp_cmd, "\n", NULL);
   else if(nvl_log == NVL_log_env)selor_superstrncpy(buffer_log, TAM_buffer_log, "(PID ", fpid, ") -> ", host.ip, " ", ptr_aux1, NULL);
   else {
    // Gravando a linha do comando passado e resposta enviada via socket, no log
    log_rettempo();
    posi_buffer=selor_superstrncpy(buffer_log, TAM_buffer_log, "(PID ", fpid, ") <- ", host.ip, " ", smtp_cmd, "\n", NULL);
    strncpy(&buffer_log[posi_buffer], buffer_log_completo, 21);
    selor_superstrncpy(&buffer_log[posi_buffer+21], TAM_buffer_log-posi_buffer+21, " (PID ", fpid, ") -> ", host.ip, " ", ptr_aux1, NULL);
   }
   log_msg(DEF_central_log);
   if(DEF_refused_log[0] && (nvl_log & NVL_log_env) && num_smtp_resp != MSG_mail_accepted){
    log_resumo_tipo[2]=0;
    if(from.local)log_resumo_tipo[0]='L';            // FROM -> Local  
    else if(from.dominio)log_resumo_tipo[0]='R';     // FROM -> Remoto 
    else log_resumo_tipo[0]='A';                     // FROM -> Anonimo
    if(atual_rcpt->local)log_resumo_tipo[1]='L';     // TO -> Local  
    else log_resumo_tipo[1]='R';                     // TO -> Remoto 
    selor_superstrncpy(buffer_log, TAM_buffer_log, log_resumo_tipo, " IP:", host.ip, " from:<", from.email, "> to:<", atual_rcpt->email, "> resp:", ptr_aux1, NULL);
    log_msg(DEF_refused_log);
   }
  }
 }
 R_REPLY[0]=0;
}

//======================================================
// Funcao para codificar um email no formato SRS        
// Retornos:                                            
// 0 = Sucesso                                          
// 1 = Endereco invalido                                
// 2 = Erro. Email original muito grande                
char srs_encode(char *ret_buffer, int tam_ret_buffer, char *email, char *novo_dominio, char *chave, unsigned char shift_carac, unsigned int srs_timer){
 char buffer_local[TAM_endereco*2];
 char senha_md5[16];
 char *dominio;
 char *fim_usuario;
 char *ptr_aux1;
 int cont1=0;
 
 if((dominio=strchr(email, '@'))){
  if(!cmp_str(email, "srs", 3) && email[4] == '+' && (email[3] == '0' || email[3] == '1')){
   // -- Ja tem SRS --
   if(email[3] == '0'){
    // -- SRS0 --
    if(strlen(email)+strlen(novo_dominio)+1 > tam_ret_buffer)return 2;
    *dominio=0;
    sprintf(ret_buffer, "SRS1+%s=%s@%s", &dominio[1], &email[5], novo_dominio);
    *dominio='@';
   }else if(email[3] == '1'){
    // -- SRS1 --
    *dominio=0;
    if(strlen(email)+strlen(novo_dominio)+1 > tam_ret_buffer)return 2;
    sprintf(ret_buffer, "%s@%s", email, novo_dominio);
    *dominio='@';
   }
  }else {
   // -- Nao tem SRS --
   if(strlen(email)+23 > tam_ret_buffer)return 2;
   srs_timer>>=15;
   cont1=snprintf(buffer_local, sizeof(buffer_local), "%s%s%u", email, chave,srs_timer);
   md5(buffer_local, cont1, senha_md5);
   ptr_aux1=buffer_local;
   for(cont1=0; cont1<16; cont1++)*ptr_aux1++=srs_caracs[((senha_md5[cont1] & 0xFF)+shift_carac) & 0xFF];
   buffer_local[16]=0;
   //for(cont1=0; cont1<16; cont1=cont1+2)*ptr_aux1++=srs_caracs[((senha_md5[cont1]^senha_md5[cont1+1] & 0xFF)+shift_carac) & 0xFF];
   //buffer_local[8]=0;
   fim_usuario=dominio;
   *fim_usuario=0;
   dominio++;
   sprintf(ret_buffer, "SRS0+%s=%s=%s@%s", buffer_local, dominio, email, novo_dominio);
   *fim_usuario='@';
  }
 }else return 1;
 return 0;
}

//======================================================
// Funcao para decodificar um email no formato SRS      
// ret_buffers:                                            
// 0 = Sucesso                                          
// 1 = Endereco invalido                                
char srs_decode(char *ret_buffer, int tam_ret_buffer, char *email, char *chave, unsigned char shift_carac, unsigned int srs_timer){
 char buffer_local[TAM_endereco*2];
 char senha_md5[16];
 char *dominio;
 char *fim_dominio;
 char *usuario;
 char *fim_usuario;
 char *hash;
 char *fim_hash;
 int cont1=0;
 int cont2=0;
 
 
 if((dominio=strchr(email, '@'))){
  *ret_buffer=0;
  if(dominio-email < 28)return 1;
  if(!cmp_str(email, "SRS", 3) && email[4] == '+' ){
   if(email[3] == '0'){
    hash=&email[5];
    if(!(fim_hash=strchr(hash, '=')))return 1;
    *fim_hash=0;
    dominio=fim_hash+1;
    if(!(fim_dominio=strchr(dominio, '='))){
     *fim_hash='=';
     return 1;
    }
    *fim_dominio=0;
    usuario=fim_dominio+1;
    if(!(fim_usuario=strchr(usuario, '@'))){
     *fim_dominio='=';
     *fim_hash='=';
     return 1;
    }
    *fim_usuario=0;
    snprintf(ret_buffer, tam_ret_buffer, "%s@%s", usuario, dominio);
    *fim_usuario='@';
    *fim_dominio='=';
    srs_timer>>=15;
    for(cont2=0; cont2<2; cont2++){
     cont1=snprintf(buffer_local, sizeof(buffer_local), "%s%s%u", ret_buffer, chave, srs_timer);
     md5(buffer_local, cont1, senha_md5);
     dominio=buffer_local;
     for(cont1=0; cont1<16; cont1++)*dominio++=srs_caracs[((senha_md5[cont1] & 0xFF)+shift_carac) & 0xFF];
     buffer_local[16]=0;
     if(!strcmp(hash, buffer_local)){
      *fim_hash='=';
      return 0;
     }
     srs_timer--;   // Para testar com o tempo anterior
    }
    *ret_buffer=0;
    *fim_hash='=';
    return 1;
   }else if(email[3] == '1'){
    dominio=&email[5];
    if(!(usuario=strchr(email, '=')))return 1;
    *usuario=0;
    if(!(fim_usuario=strchr(usuario+1, '@'))){
     *usuario='=';
     return 1;
    }
    *fim_usuario=0;
    snprintf(ret_buffer, tam_ret_buffer, "SRS0+%s@%s", usuario+1, dominio);
    *fim_usuario='@';
    *usuario='=';
    return 0;
   }
  }
 }
 return 1;  // Endereco invalido
}




//======================================================
// Copia o end. passado pelo 'mail from' ou 'rcpt to'   
// para uma estrutura 'dados_email' apontada.           
// Em 'tam'  retornado o valor de 'SIZE' passado via   
// comando smtp 'mail from'                             
// Retornos:                                            
// 0 = Email contem <>                                  
// >0 = Email nao contem <>                             
// -1 = Endereco invalido                               
char smtp_dados_email(char *smtp_dados, struct dados_email *struct_email, unsigned int *tam){
 char *ptr_aux1=NULL;
 char *ptr_aux2=NULL;
 char byte=0;
 char chevron=2;
 
 
 smtp_dados=pular_carac(smtp_dados, ' ');
 if(*smtp_dados == '<'){
  smtp_dados++;
  chevron--;
 }
 if((ptr_aux1=strchr(smtp_dados, '>')))chevron--;
 else ptr_aux1=strchr(smtp_dados, ' ');
 if(ptr_aux1){
  ptr_aux2=ptr_aux1;
  byte=*ptr_aux1;
  *ptr_aux1=0;
  if(tam){      // Analise do SIZE foi pedido
   *tam=0;
   ptr_aux1=pular_carac(++ptr_aux1, ' ');
   if(ptr_aux1 && !cmp_str(ptr_aux1, "size=", 5)){
    ptr_aux1+=5;
    *tam=atoi(ptr_aux1);
   }
  }
 }
 if(selor_segcp(struct_email->email, smtp_dados) > 1)return -1;   // --> Endereco invalido. 2 arrobas ou mais
 if(ptr_aux2)*ptr_aux2=byte;              // Restaurando o byte do comando smtp passado porque  usado no LOG
 if((struct_email->dominio=strchr(struct_email->email, '@')))struct_email->dominio++;
 else if(DEF_df_domain[0]){
  // Definindo o dominio padrao
  strcat(struct_email->email, "@");
  strcat(struct_email->email, DEF_df_domain);
  if((struct_email->dominio=strchr(struct_email->email, '@')))struct_email->dominio++;
 }
 return chevron;
}

//======================================================
// Funcao para gravar um arquivo de lista               
// Retornos:                                            
// 0 = Sucesso                                          
// 1 = Sem espaco em disco                              
char gravar_lista(unsigned int local_num_rcpts, FILE *local_arq, char *cam_arq, unsigned int num_tentativas, unsigned int num_report_erros){
 char status_local[2]={0,0};
 char buffer_local[TAM_buffer];
 char *ptr_aux1=buffer_local;
 unsigned int tam_buffer=0;
 unsigned int tam_arq=0;
 unsigned int cont1=0;
 unsigned int cont2=0;
 unsigned long *op_numerica;
 
 fseek(local_arq, 0, 0);
 ptr_aux1+=tam_buffer=sprintf(ptr_aux1, "<%s> %d %d\n", from.email, num_tentativas, num_report_erros);
 if(num_ops){
  // Gravando Def!
  for(cont1=0; cont1<num_ops; cont1++){
   if(def_ops_val[def_ops[cont1]] & 1){    // --> Para impedir de gravar opcoes inuteis dentro do arquivo. Apenas opcoes marcados com OP_DEF_ON
    if(def_ops[cont1] < Log_lvl)ptr_aux1+=cont2=selor_superstrncpy(ptr_aux1, TAM_buffer, "Def:", nome_op[def_ops[cont1]], " \"", (char *)ptr_op[def_ops[cont1]], "\"\n", NULL);
    else {
     op_numerica=ptr_op[def_ops[cont1]];
     ptr_aux1+=cont2=selor_superstrncpy(ptr_aux1, TAM_buffer, "Def:", nome_op[def_ops[cont1]], " ", itoa(*op_numerica, 0), "\n", NULL);
    }
    tam_buffer+=cont2;
    if(tam_buffer > TAM_buffer/2){
     // Descarrengado buffer no disco
     tam_arq+=tam_buffer;                  // Calculando tamanho do arquivo
     if((fwrite(buffer_local, 1, tam_buffer, local_arq)) != tam_buffer)return 1;
     ptr_aux1=buffer_local;
     tam_buffer=0;
    }
   }
  }
 }
 if(R_HADD[0] && arq_data_criado){
  // Adicionando HADD 
  ptr_aux1+=cont2=selor_superstrncpy(ptr_aux1, TAM_hadd, "Hadd:", R_HADD, NULL);
  tam_buffer+=cont2;
 }
 for(cont1=0; cont1<local_num_rcpts; cont1++){
  status_local[0]=to[cont1].status;
  ptr_aux1+=cont2=selor_superstrncpy(ptr_aux1, TAM_buffer, "[", status_local, " ", to[cont1].email, "]\n", NULL);
  tam_buffer+=cont2;
  if(to[cont1].resp[0]){
   // Gravando ultima resposta
   ptr_aux1+=cont2=selor_superstrncpy(ptr_aux1, TAM_email_resp+10, "Resp:", to[cont1].resp, "\n", NULL);
   tam_buffer+=cont2;
  }
  if(to[cont1].dest[0]){
   // Gravando Mx_Dest/Dlocal diferente
   ptr_aux1+=cont2=selor_superstrncpy(ptr_aux1, TAM_email_resp+10, "Dest:", to[cont1].dest, "\n", NULL);
   tam_buffer+=cont2;
  }
  if(to[cont1].auth[0]){
   // Gravando Mx_auth diferente
   ptr_aux1+=cont2=selor_superstrncpy(ptr_aux1, TAM_email_resp+10, "Mx_auth:", to[cont1].auth, "\n", NULL);
   tam_buffer+=cont2;
  }
  if(to[cont1].so_mark){
   // Gravando So_mark pedido
   ptr_aux1+=cont2=selor_superstrncpy(ptr_aux1, TAM_email_resp+10, "So_mark:", itoa(to[cont1].so_mark, 0), "\n", NULL);
   tam_buffer+=cont2;
  }
  if(to[cont1].mx_source[0]){
   // Gravando Mx_source
   ptr_aux1+=cont2=selor_superstrncpy(ptr_aux1, TAM_email_resp+10, "Mx_source:", to[cont1].mx_source, "\n", NULL);
   tam_buffer+=cont2;
  }
  if(tam_buffer > TAM_buffer/2){
   // Descarrengado buffer no disco
   tam_arq+=tam_buffer;           // Calculando tamanho do arquivo
   if((fwrite(buffer_local, 1, tam_buffer, local_arq)) != tam_buffer)return 1;
   ptr_aux1=buffer_local;
   tam_buffer=0;
  }
 }
 tam_arq+=tam_buffer;             // Calculando tamanho do arquivo
 if((fwrite(buffer_local, 1, tam_buffer, local_arq)) != tam_buffer)return 1;  // Descarregando buffer restante no disco
 fflush(local_arq);
 truncate(cam_arq, tam_arq);      // Truncando tamanho do arquivo
 return 0;
}

//======================================================
// Funcao para responder com erro temporario e fazer log
// do erro do modulo                                    
void tratar_erro_modulo(int mod_indice){

 num_smtp_resp=MSG_internal_error;
 smtp_resp(NULL, DEF_log_lvl, NVL_log_rec_env);
 selor_superstrncpy(buffer_log, TAM_buffer_log, "MODULE: (PID ", fpid, ") ", host.ip, " - ", mod_msg_error[mod_indice](), "\n", NULL);
 log_msg(DEF_warning_log);
}

//======================================================
// Funcao para fazer cache das consultas de dominios    
// Retornos:                                            
// 0 = Dominio externo                                  
// 1 = Dominio interno                                  
// -1 = Erro do modulo                                  
char cache_mod_dom_local(char *dominio, char *ret_dom, unsigned int *indice_achado){
 char cont1=0;
 
 if(!dominio)return 1; // Sem dominio, indica usuario de sistema interno
 while(1){
  *indice_achado=0;
  if(!cmp_str_vars(dominio, ult_dom_ext))return 0;        // Dominio EXTERNO
  else if(!cmp_str_vars(dominio, ult_dom_int))return 1;   // Dominio INTERNO
  // Checando por localhost, Server_name
  if(!strcmp(dominio, "localhost") || !cmp_str_vars(dominio, DEF_server_name))return 1;
  // Checando por IPs locais
  if(*dominio >= '0' && *dominio <= '9' && ip_local(inet_addr(dominio)))return 1;
  //if(!(ret_val=mod_dom_local[0](dominio, ret_dom)))strcpy(ult_dom_ext, dominio);      // Consultando modulo e fazendo cache
  ret_dom[0]=0;
  while(!(ret_val=mod_dom_local[modseq_domain[*indice_achado]](dominio, ret_dom)) && (modseq_domain[++*indice_achado] != 0xFF)); // Consultando os modulos
  if(!*ret_dom)break;
  if(cont1 >= MAX_adomain_resolvs){   // Numero maximo de 'resolvimentos' de alias.
   ret_val=0;
   break;
  }
  selor_strncpy(dominio, ret_dom, TAM_dominio);
  cont1++;
 }
 if(ret_val < 0)return ret_val;  // Possivel erro no modulo
 if(cont1)selor_strncpy(ret_dom, dominio, TAM_dominio);       // Para alertar que o dominio foi mudado
 if(ret_val == 1)selor_strncpy(ult_dom_int, dominio, sizeof(ult_dom_int)); // Fazendo cache
 else selor_strncpy(ult_dom_ext, dominio, sizeof(ult_dom_ext));            // fazendo cache
 return ret_val;    // Possivel erro no modulo
}

//======================================================
// Funcao que coloca em um rcpt uma mensagem de status  
void resp_rcpt(int email_index, char *ip, char *resp){

 str_arruma_crlf(resp);
 selor_superstrncpy(to[email_index].resp, TAM_email_resp, "[", ip, "] -> ", resp, NULL);
 if(*resp == '4')to[email_index].status=STATUS_ERRO_TEMP;
 else if(*resp == '2')to[email_index].status=STATUS_ENTREGUE;
 else to[email_index].status=STATUS_ERRO;
}

//======================================================
// Funcao que coloca na lista de rcpts uma mensagem de  
// status                                               
void resp_rcpts(int email_index, char *ip, char *resp, char *id){
 char status;
 int prim_index=0;
 
 if(*resp == '2'){
  status=STATUS_ENTREGUE;
  prim_index=email_index;
 }
 else if(*resp == '4')status=STATUS_ERRO_TEMP;
 else status=STATUS_ERRO;
 str_arruma_crlf(resp);
 while(email_index >= 0){
  if(to[email_index].status == STATUS_PENDENTE){
   selor_superstrncpy(to[email_index].resp, TAM_email_resp, "[", ip, "] -> ", resp, NULL);
   to[email_index].status=status;
   if(status == STATUS_ENTREGUE){
    // Fazendo log de resumo
    if(prim_index == email_index)selor_superstrncpy(buffer_log, TAM_buffer_log, log_resumo_tipo, " ID:", id, " from:<", from.email, "> to:<", to[email_index].email, "> size:", log_resumo_size, " transf:", log_resumo_size, "\n", NULL);
    else selor_superstrncpy(buffer_log, TAM_buffer_log, log_resumo_tipo, " ID:", id, " from:<", from.email, "> to:<", to[email_index].email, "> size:", log_resumo_size, " transf:0\n", NULL);
    log_msg(DEF_sumary_log);
   }
  }else if(to[email_index].status == STATUS_TARPIT)to[email_index].status=STATUS_PENDENTE;
  email_index=mx_index[email_index];
 }
}

//======================================================
// Funcao para entregar um email localmente             
// Retorna o valor da funcao system                     
int dlocal(char *dlocal_str, char *arq_email, char *id, struct dados_email *dlocal_from, struct dados_email *dlocal_to){
 pid_t deliver;
 char *parametros[20];
 char tipo_entrega=1;           // 0 = Via PIPE. 1 = Parametro
 char buffer_local[TAM_buffer];
 char buffer_local2[TAM_buffer];
 char *ptr_aux1;
 char *ptr_aux2;
 char *fim;
 char byte[4]={' ',0,0,0};
 int email;
 int ctrl_pipe[2];
 int resp=0;
 int cont1=0;
 int cont2=0;
 
 if(*dlocal_str && (*dlocal_str == '/' || *dlocal_str == '|')){
  if(DEF_adlocal_allowed[0]){
   // Verificando se o dlocal forcado esta autorizado
   if(strchr(dlocal_str, '&') || strchr(&dlocal_str[1], '|'))goto Dlocal_denied;
   ptr_aux1=ptr_aux2=DEF_adlocal_allowed;
   while(*ptr_aux2){
    // Procurando programas permitidos em DEF_adlocal_allowed separados por espacos
    if(!(ptr_aux2=strchr(ptr_aux2, ' ')))goto Dlocal_denied;
    ptr_aux2++;
    if(!strncmp(dlocal_str, ptr_aux1, ptr_aux2-ptr_aux1))goto Dlocal_ok;
    ptr_aux1=ptr_aux2;
   }
   Dlocal_denied:
   snprintf(buffer_log, TAM_buffer_log, "(%s) Dlocal DENIED [%s]\n", id, dlocal_str);
   log_msg(DEF_central_log);
   return -1;
  }
  Dlocal_ok:
  ptr_aux1=dlocal_str;
 }else ptr_aux1=DEF_dlocal;
 if(*ptr_aux1 == '|'){
  ptr_aux1++;
  byte[0]=byte[2]=' ';
  byte[1]='<';
 }
 ptr_aux2=buffer_local;
 fim=ptr_aux2+TAM_buffer;
 
 while(*ptr_aux1 && ptr_aux2 < fim){     // Copiando string traduzindo as variaveis
  if(*ptr_aux1 == '%'){
   // -- Variavel --
   ptr_aux1++;
   if(*ptr_aux1 == 'l'){        // -- Usuario LOCAL --
    ptr_aux2+=selor_strncpy(ptr_aux2, dlocal_to->email, TAM_buffer - (ptr_aux2-buffer_local));
   }else if(*ptr_aux1 == 'r'){  // -- Email do destinatario ---
    ptr_aux2+=selor_strncpy(ptr_aux2, dlocal_to->email, TAM_buffer - (ptr_aux2-buffer_local));
   }else if(*ptr_aux1 == 'f'){  // -- Email do remetente ---
    ptr_aux2+=selor_strncpy(ptr_aux2, dlocal_from->email, TAM_buffer - (ptr_aux2-buffer_local));
   }else if(*ptr_aux1 == 'c'){  // -- Dominio do destinatario ---
    if(dlocal_to->dominio)ptr_aux2+=selor_strncpy(ptr_aux2, dlocal_to->dominio, TAM_buffer - (ptr_aux2-buffer_local));
    else ptr_aux2+=selor_strncpy(ptr_aux2, "(null)", TAM_buffer - (ptr_aux2-buffer_local));
   }else if(*ptr_aux1 == 'o'){  // -- Dominio do remetente ---
    if(dlocal_from->dominio)ptr_aux2+=selor_strncpy(ptr_aux2, dlocal_from->dominio, TAM_buffer - (ptr_aux2-buffer_local));
    else ptr_aux2+=selor_strncpy(ptr_aux2, "(null)", TAM_buffer - (ptr_aux2-buffer_local));
   }else if(*ptr_aux1 == 'd'){  // -- ID da mensagem ---
    ptr_aux2+=selor_strncpy(ptr_aux2, id, TAM_buffer - (ptr_aux2-buffer_local));
   }else if(*ptr_aux1 == 'p'){  // -- Usuario antes do @ no destinatario ---
    if(dlocal_to->dominio){
     dlocal_to->dominio[-1]=0;
     ptr_aux2+=selor_strncpy(ptr_aux2, dlocal_to->email, TAM_buffer - (ptr_aux2-buffer_local));
     dlocal_to->dominio[-1]='@';
    }else ptr_aux2+=selor_strncpy(ptr_aux2, "(null)", TAM_buffer - (ptr_aux2-buffer_local));
   }else if(*ptr_aux1 == 'm'){  // -- Usuario antes do @ no remetente ---
    if(dlocal_from->dominio){
     dlocal_from->dominio[-1]=0;
     ptr_aux2+=selor_strncpy(ptr_aux2, dlocal_from->email, TAM_buffer - (ptr_aux2-buffer_local));
     dlocal_from->dominio[-1]='@';
    }else ptr_aux2+=selor_strncpy(ptr_aux2, "(null)", TAM_buffer - (ptr_aux2-buffer_local));;
   }else if(*ptr_aux1 == 'C'){  // -- Diretorio HOME do destinatario ---
    if(rcpt_info.home[0])ptr_aux2+=selor_strncpy(ptr_aux2, rcpt_info.home, TAM_buffer - (ptr_aux2-buffer_local));
    else {
     // Nao existe informacao do home. Vamos pega-la
     //mod_user_local[0](dlocal_to->email, mod_user_info, MOD_user_ncampos);
     cont1=0;
     while(!(ret_val=mod_user_local[modseq_mail[cont1]](dlocal_to->email, mod_user_info, MOD_user_ncampos)) && (modseq_mail[++cont1] != 0xFF));
     ptr_aux2+=selor_strncpy(ptr_aux2, mod_user_info[MOD_user_home], TAM_buffer - (ptr_aux2-buffer_local));
    }
   }else if(*ptr_aux1 == '%')*ptr_aux2++=*ptr_aux1;  // -> Caractere %
  }else *ptr_aux2++=*ptr_aux1;
  ptr_aux1++;
 }
 selor_superstrncpy(ptr_aux2, TAM_buffer - (ptr_aux2-buffer_local), byte, arq_email, NULL);
 
 if(DEF_dlocal_max_procs > 0)semop(semid, &sm_travar, 1);
 
 if(!DEF_dlocal_type){
  strcpy(buffer_local2, buffer_local);
  parametros[cont2++]=buffer_local2;
  ptr_aux1=buffer_local2;
  ptr_aux2=strchr(buffer_local2, '<');
  if(ptr_aux2){
   tipo_entrega=0;        // Via PIPE
   *ptr_aux2=' ';
  }
  while(ptr_aux1 && *ptr_aux1){
   ptr_aux1=strchr(ptr_aux1, ' ');
   if(ptr_aux1){
    *ptr_aux1++=0;
    ptr_aux1=pular_carac(ptr_aux1, ' ');
   }
   parametros[cont2++]=ptr_aux1;
  }
  if(!tipo_entrega){
   cont2--;                        // Removendo o parametro referente ao caminho do arquivo de fila
   parametros[cont2-1]=NULL;       // pois sera entregue via PIPE                                  
   if(pipe(ctrl_pipe) == -1){
    snprintf(buffer_log, TAM_buffer_log, "(%s) 255:0 <- [%s] PIPE Create ERROR\n", id, buffer_local);
    log_msg(DEF_central_log);
    return 255;
   }
  }
  deliver=fork();
  if(!deliver){
   close(ctrl_pipe[1]);
   dup2(ctrl_pipe[0], 0);
   execv(parametros[0], parametros);
   close(ctrl_pipe[0]);
   exit(1);
  }
  if(!tipo_entrega){
   // Entregando via PIPE
   close(ctrl_pipe[0]);
   email=open(arq_email, O_RDONLY);
   do{
    cont2=read(email, buffer_local2, TAM_buffer);
    if(cont2 > 0)write(ctrl_pipe[1], buffer_local2, cont2);
   }while(cont2 == TAM_buffer);
   close(email);
   close(ctrl_pipe[1]);
  }
  waitpid(deliver, &resp, 0);
 }else resp=system(buffer_local);   // -- PADRAO
 
 if(DEF_dlocal_max_procs > 0)semop(semid, &sm_liberar, 1);
 
 snprintf(buffer_log, TAM_buffer_log, "(%s) %u:%u <- [%s]\n", id, resp & 0xFF, resp >> 8, buffer_local);
 //selor_superstrncpy(buffer_log, TAM_buffer_log, "(", id, ") ", itoa(resp & 0xFF, 0), ":", itoa(resp >> 8, 0), " <- [", buffer_local, "]\n", NULL);
 log_msg(DEF_central_log);
 return resp;
}

//======================================================
// Funcao para enviar o conteudo de um arquivo pelo     
// atual socket conectado a socket_remoto               
// Retornos:                                            
// 0 = Sucesso                                          
// -1 = Socket error                                    
char data_env_arq(FILE *arq_email, char finalizar){
 char sinal=0;
 int b_lidos=0;
 int b_env=0;
 unsigned int cont1=0;
 unsigned int cont2=0;
 
 cont2=0;
 while(!feof(arq_email)){
  b_lidos=fread(buffer, 1, TAM_buffer, arq_email);
  for(cont1=0; cont1<b_lidos; cont1++){
   if(cont2 >= (TAM_buffer-10)){
    // Buffer cheio. Vamos envia-lo
    linha[cont2]=0;
    b_env=selor_send(socket_remoto, linha, cont2, 0);
    if(b_env != cont2)return -1;
    cont2=0;
   }
   if(buffer[cont1] == 10){
    linha[cont2++]=13;
    linha[cont2++]=10;
    sinal=1;
   }else {
    if(sinal){
     if(buffer[cont1] == '.'){
      linha[cont2++]='.';
      linha[cont2++]='.';
     }else linha[cont2++]=buffer[cont1];
     sinal=0;
    }else linha[cont2++]=buffer[cont1];
   }
  }
 }
 // Enviando o resto do buffer
 if(finalizar){
  strcpy(&linha[cont2], "\r\n.\r\n");
  cont2+=5;
 }
 b_env=selor_send(socket_remoto, linha, cont2, 0);
 if(b_env != cont2)return -1;
 return 0;
}


//======================================================
// Funcao para executar os primeiros comnados SMTP:     
// helo,starttls,auth e mail from                       
// Retornos em bits:                                    
// 0 = sucesso                                          
// bit 0 (0x1) = Cnx Perdida(Interrompe a funcao)       
// bit 1 (0x2) = Erro 4xx ou 5xx (Interrompe a funcao)  
// bit 2 (0x4) = Host remoto nao suporta TLS            
// bit 3 (0x8) = Erro no TLS                            
// bit 4 (0x10)= Erro nao autenticacao                  
char cnx_smtp_init(int num_socket, char *id, char *ip, char *ptr_auth, char *ptr_from, char fazer_log){
 char auth_plain=0;
 char *auth_user;
 char *auth_pass;
 char retorno=0;
 int b_lidos;
 int cont1=0;
 int cont2;
 
 CMD_helo:
 b_lidos=selor_superstrncpy(buffer, TAM_buffer, "ehlo ", DEF_server_name, "\r\n", NULL);
 Novo_cmd_helo:
 selor_send(num_socket, buffer, b_lidos, 0);
 b_lidos=selor_smtp_recv(num_socket, linha, TAM_buffer, 0);
 if(b_lidos < 1)return 1;
 if(linha[0] == '4')return 0x2; // Resposta de erro temporario 4xx
 if(linha[0] != '2'){
  if(cont1)return 0x2; // Resposta de erro permanente 5xx
  //Talves ele nao tenha o comando ehlo. Vamos tentar um helo normal
  b_lidos=selor_superstrncpy(buffer, TAM_buffer, "helo ", DEF_server_name, "\r\n", NULL);
  cont1=1;
  goto Novo_cmd_helo;
 }
 // Ativando TLS se foi solicitado
 if(DEF_tls_delivery && !tls_ativo){
  // Verificando se o servidor tem TLS
  if(proc_str(linha, "250-STARTTLS\0") || proc_str(linha, "250 STARTTLS\0")){
   // Iniciando TLS
   selor_send(num_socket, "starttls\r\n", 10, 0);
   if(fazer_log){
    selor_superstrncpy(buffer_log, TAM_buffer_log, "(", id, ") -> ", ip, " starttls\n", NULL);
    log_msg(DEF_central_log);
   }
   b_lidos=selor_smtp_recv(num_socket, buffer, TAM_buffer, 0);
   if(b_lidos < 1 || buffer[0] != '2')return 1;
   temp=0;
   alarm(DEF_timeout);
   temp=tls_connect(num_socket);
   alarm(0);
   if(!temp){
    tls_ativo=1;
    goto CMD_helo;
   }
   else {
    if(fazer_log){
     selor_superstrncpy(buffer_log, TAM_buffer_log, "(PID ", fpid, ") ERROR: ", tls_error(), "\n", NULL);
     log_msg(DEF_warning_log);
    }
    retorno|=0x8; //TLS error
   }
  }else retorno|=0x4; //Host remoto nao possui TLS
 }
 if(ptr_auth && *ptr_auth){
  // -- Autenticando-se --
  cont1=selor_strncpy(&linha[1], ptr_auth, TAM_email_auth);
  cont1++;
  auth_user=&linha[1];
  if((auth_pass=strchr(auth_user, ' '))){
   *auth_pass++=0;
   if(proc_str(linha, "PLAIN"))auth_plain=1;
   else if(!proc_str(linha, "LOGIN"))auth_plain=1;
   if(auth_plain){
    // -- AUTH PLAIN --
    linha[0]=0;
    strcpy(buffer, "auth plain ");
    cont2=base64_cod(&buffer[11], linha, cont1);
    strcat(buffer, "\r\n");
    cont2+=13;        // --> 11(auth plain ) + 2(\r\n) = 13
    selor_send(num_socket, buffer, cont2, 0);
    if(fazer_log){
     selor_superstrncpy(buffer_log, TAM_buffer_log, "(", id, ") -> ", ip, " auth plain ********\n", NULL);
     log_msg(DEF_central_log);
    }
    b_lidos=selor_smtp_recv(num_socket, buffer, TAM_buffer, 0);
    if(fazer_log)selor_superstrncpy(buffer_log, TAM_buffer_log, "(", id, ") <- ", ip, " ", buffer, NULL);
   }else {
    // -- AUTH LOGIN --
    selor_send(num_socket, "auth login\r\n", 12, 0);
    b_lidos=selor_smtp_recv(num_socket, buffer, TAM_buffer, 0);
    if(b_lidos < 1)return retorno | 1;
    if(buffer[0] == '3'){
     cont2=base64_cod(buffer, auth_user, strlen(auth_user));
     strcat(buffer, "\r\n");
     cont2+=2;
     selor_send(num_socket, buffer, cont2, 0);
     b_lidos=selor_smtp_recv(num_socket, buffer, TAM_buffer, 0);
     if(b_lidos < 1)return retorno | 1;
     if(buffer[0] == '3'){
      cont2=base64_cod(buffer, auth_pass, strlen(auth_pass));
      strcat(buffer, "\r\n");
      cont2+=2;
      selor_send(num_socket, buffer, cont2, 0);
      if(fazer_log){
       selor_superstrncpy(buffer_log, TAM_buffer_log, "(", id, ") -> ", ip, " auth login ********\n", NULL);
       log_msg(DEF_central_log);
      }
      b_lidos=selor_smtp_recv(num_socket, buffer, TAM_buffer, 0);
     }
    }
   }
   if(b_lidos < 1)return retorno | 1;
   if(buffer[0] != '2'){
    // -- Senha Recusada. Mas vamos ignorar e tentar mandar o email assim mesmo --
    if(fazer_log){
     selor_superstrncpy(buffer_log, TAM_buffer_log, "(", id, ") <- ", ip, " ", buffer, NULL);
     log_msg(DEF_central_log);
    }
    retorno|=0x10; //Erro na autenticacao
   }
  }
 }
 // Passando endereco do remetente
 if(*ptr_from == '<')b_lidos=selor_superstrncpy(buffer, TAM_buffer, "mail from: ", ptr_from, "\r\n", NULL);
 else b_lidos=selor_superstrncpy(buffer, TAM_buffer, "mail from: <", ptr_from, ">\r\n", NULL);
 selor_send(num_socket, buffer, b_lidos, 0);
 if(fazer_log){
  selor_superstrncpy(buffer_log, TAM_buffer_log, "(", id, ") -> ", ip, " ", buffer, NULL);
  log_msg(DEF_central_log);
 }
 b_lidos=selor_smtp_recv(num_socket, buffer, TAM_buffer, 0);
 if(b_lidos < 1)return retorno | 1;
 if(buffer[0] != '2')retorno|=0x2; // Remetente recusado
 return retorno;
}



//======================================================
// Envia um arquivo de email diretamente ao destino     
// local ou remoto                                      
// Retornos:                                            
// 0 = Enviado com sucesso                              
// 1 = Arquivo nao encontrado                           
// 2 = Socket error                                     
// 3 = Remetente recusado                               
// 4 = Destinatario recusado                            
char sendmail(char *cam_email, char *id, struct dados_email *local_from, struct dados_email *local_to){
 FILE  *arq_email;
 struct dados_email str_mail_local;
 char *ptr_aux1;
 char ip[16];
 int temp1=0;
 unsigned int cont1=0;
 unsigned int cont2=0;
 unsigned int ipn=0;
 unsigned int prox_bloco=0;
 
 if(!cam_email || !*cam_email)return 1;
 arq_email=fopen(cam_email, "rb");
 // -- Testando por SRS --
 if(DEF_srs_key[0] && !cmp_str(local_to->email, "SRS", 3)
                   && (local_to->email[3] == '0' || local_to->email[3] == '1') && local_to->email[4] == '+'
                   && !srs_decode(buffer, TAM_endereco, local_to->email, DEF_srs_key, DEF_srs_shift, tempo)){
  if((smtp_dados_email(buffer, &srs_tmp, NULL)) == -1)return 4;       // Endereco de destinatario invalido
  local_to=&srs_tmp;
 }
 
 // -- Testando se e' local --
 if(!local_to->dominio || (cache_mod_dom_local(local_to->dominio, adomain, &cont1)) == 1){
  // ---------------
  //  Dominio local 
  // ---------------
  while(1){
   cont1=0;
   while(!(ret_val=mod_user_local[modseq_mail[cont1]](local_to->email, mod_user_info, MOD_user_ncampos)) && (modseq_mail[++cont1] != 0xFF));
   if(ret_val){
    if(mod_user_info[MOD_user_dlocal])selor_strncpy(local_to->dest, mod_user_info[MOD_user_dlocal], TAM_email_dest);
    break;   // --> Usuario encontrado
   }
   if(++cont2 > 3){
    fclose(arq_email);
    return 4;
   }
   cont1=0;
   prox_bloco=0;
   while(!(temp1=mod_ret_alias[modseq_alias[cont1]](local_to->email, ret_alias[0], 1, TAM_endereco, &prox_bloco)) && (modseq_alias[++cont1] != 0xFF));
   if(!temp1){
    if(!local_to->dominio || (cache_mod_dom_local(local_to->dominio, adomain, &cont1)) == 1)break; // Usuario nao encontrado mas o domnio e' local. Vamos tentar entregar
    local_to->dest[0]=0;
    goto Dominio_externo;
   }
   if((smtp_dados_email(ret_alias[0], &str_mail_local, NULL)) == -1)return 4;       // Endereco de destinatario invalido
   local_to=&str_mail_local;
  }
  // -- Usuario LOCAL encontrado --
  fclose(arq_email);
  cont1=0;
  while(!(ret_val=mod_test_quota[modseq_mail[cont1]](local_to->email, mod_user_info, MOD_user_ncampos)) && (modseq_mail[++cont1] != 0xFF));
  if(ret_val == 2){
   selor_superstrncpy(buffer_log, TAM_buffer_log, "(", id, ") <- 127.0.0.1 [MAILER-DAEMON] ", local_to->email, " ", DEF_msg_full_mailbox, "\n", NULL);
   log_msg(DEF_central_log);
   return 4;
  }
  Dominio_local:
  if(dlocal(local_to->dest, cam_email, id, local_from, local_to))return 4; //--> Erro na entrega local
  
 }else {
  // -----------------
  //  Dominio externo 
  // -----------------
  Dominio_externo:
  if((socket_remoto=socket(AF_INET, SOCK_STREAM, 0)) == -1){
   // Erro no socket
   selor_superstrncpy(buffer_log, TAM_buffer_log, "(PID ", fpid, ") ERROR: ", strerror(errno), "\n", NULL);
   log_msg(DEF_warning_log);
   fclose(arq_email);
   return 2;
  }
  rede_remota.sin_family=AF_INET;
  if(!local_to->dest[0]){
   // -- Resolvendo MX --
   rede_remota.sin_port=htons(PORTA_padrao);
   if(achar_mx(local_to->dominio, &cont2)){
    // MX nao encontrado. Vamos usar o dominio mesmo
    dns_num_mxzes=1;
    selor_strncpy(dns_resp_indexado[0].nome, local_to->dominio, NS_MAXDNAME);
   }
   buffer[0]=0;
   do{
    if(!(ipn=prox_mx(ip, DEF_mx_alias_ip))){
     // Sem mais MXzes. Terminar as tentativas
     Cnx_fechada:
     close(socket_remoto);
     fclose(arq_email);
     return 2;
    }
    if(ip_local(ipn)){      // Testando se o MX nao aponta para o proprio servidor
     selor_superstrncpy(buffer_log, TAM_buffer_log, "(PID ", fpid, ") WRN: The domain ", from.dominio, " returns a local IP, but it is not in local list\n", NULL);
     log_msg(DEF_warning_log);
     close(socket_remoto);
     fclose(arq_email);
     goto Dominio_local;
    }
    rede_remota.sin_addr.s_addr=ipn;
    memset(&rede_remota.sin_zero, 0, 8);
    alarm(DEF_connect_timeout);
    cont2=connect(socket_remoto, (struct sockaddr *)&rede_remota, sizeof(struct sockaddr));
    alarm(0);
   }while(cont2);
  }else {
   // -- MX Forcado --
   dns_num_ip=0;
   if((ptr_aux1=strchr(local_to->dest, ' '))){
    *ptr_aux1++=0;
    ptr_aux1=pular_carac(ptr_aux1, ' ');
    rede_remota.sin_port=htons(atoi(ptr_aux1));
   }else rede_remota.sin_port=htons(PORTA_padrao);
   do{
    if(!(ipn=resolver_dom(local_to->dest, ip, dns_num_ip)))goto Cnx_fechada;       // Nao foi possivel resolver o endereco
    dns_num_ip++;
    if(ip_local(ipn)){
     fclose(arq_email);
     goto Dominio_local;
    }
    rede_remota.sin_addr.s_addr=ipn;
    memset(&rede_remota.sin_zero, 0, 8);
    alarm(DEF_connect_timeout);
    cont2=connect(socket_remoto, (struct sockaddr *)&rede_remota, sizeof(struct sockaddr));
    alarm(0);
   }while(cont2);
  }
  temp1=selor_smtp_recv(socket_remoto, buffer, TAM_buffer, 0);
  if(temp1 < 1 || buffer[0] != '2')goto Cnx_fechada;
  if((cnx_smtp_init(socket_remoto, id, ip, local_to->auth, local_from->email, 0) & 0x3))goto Cnx_fechada;
  temp1=selor_superstrncpy(buffer, TAM_buffer, "rcpt to: <", local_to->email, ">\r\n", NULL);
  selor_send(socket_remoto, buffer, temp1, 0);
  temp1=selor_smtp_recv(socket_remoto, buffer, TAM_buffer, 0);
  if(temp1 < 1 || buffer[0] != '2')goto Cnx_fechada;
  selor_send(socket_remoto, "data\r\n", 6, 0);
  temp1=selor_smtp_recv(socket_remoto, buffer, TAM_buffer, 0);
  if(temp1  < 1 || (buffer[0] != '3' && buffer[1] != '5' && buffer[2] != '4'))goto Cnx_fechada;
  // -- Enviando conteudo do email --
  if(data_env_arq(arq_email, 1))goto Cnx_fechada;
  temp1=selor_smtp_recv(socket_remoto, buffer, TAM_buffer, 0);
  fclose(arq_email);
  close(socket_remoto);
 }
 return 0;
}

//======================================================
// Funcao para enviar um email ao POSTMASTER avisando   
// que uma mensagem nao pode ser reportada              
void postmaster_report(char *id, FILE *arq_lid){
 FILE *arq_erro;
 char cam_tmp_email[TAM_cam_email];
 int b_lidos=0;
 char tmp;

 if(!DEF_postmaster[0])return;
 selor_superstrncpy(cam_tmp_email, TAM_buffer, DEF_temp_path, "/e", id, ".tmp", NULL);
 if(!(arq_erro=fopen(cam_tmp_email, "wb"))){
  // Erro criando arquivo temporario
  selor_superstrncpy(buffer_log, TAM_buffer_log, "(PID ", fpid, ") ERROR: Could not create the temporary file for error report [", linha, "]\n", NULL);
  log_msg(DEF_warning_log);
  return;
 }
 rfc_date(linha, tempo);      // Timezone ja  embutido por esta funcao
 b_lidos=selor_superstrncpy(buffer, TAM_buffer, "Return-Path: ", DEF_report_from, "\nReceived: from localhost (localhost[127.0.0.1])\n\tby ", DEF_server_name, " SMTP SELOR v", VERSAO, " id ", id, "\n\tfor <", DEF_postmaster, ">; ", linha, "\n", NULL);
 fwrite(&buffer, 1, b_lidos, arq_erro);
 b_lidos=selor_superstrncpy(buffer, TAM_buffer, "Date: ", linha, "\nFrom: Mailer-Daemon ", DEF_report_from, "\nTo: ", DEF_postmaster, "\nSubject: Return error\n\n", NULL);
 fwrite(&buffer, 1, b_lidos, arq_erro);
 b_lidos=selor_superstrncpy(buffer, TAM_buffer, "The sender (", from.email, ") cannot be informed about this error\nBelow is the contents of the file:\n\nID (", id, ")\n--------------------\n", NULL);
 fwrite(&buffer, 1, b_lidos, arq_erro);
 fseek(arq_lid, 0, 0);
 while(!feof(arq_lid)){
  b_lidos=fread(buffer, 1, TAM_buffer, arq_lid);
  fwrite(buffer, 1, b_lidos, arq_erro);
 }
 if(DEF_cp_errors){
  // Copiando arquivos para o DEF_cp_errors_path
  b_lidos=selor_superstrncpy(buffer, TAM_buffer, "\n\n--------------------\nA copy of this ID was made in ", DEF_cp_errors_path, "\n", NULL);
  fwrite(buffer, 1, b_lidos, arq_erro);
  selor_superstrncpy(buffer, TAM_buffer, "cp ", DEF_queue_path, "/*", id, " ", DEF_cp_errors_path, NULL);
  system(buffer);
 }else fwrite("\n\n--------------------\n", 1, 23, arq_erro);
 fclose(arq_erro);
 
 // Entregando a mensagem ao postmaster
 smtp_dados_email(DEF_report_from, &localhost, NULL);
 smtp_dados_email(DEF_postmaster, &tmp_email, NULL);
 tmp=sendmail(cam_tmp_email, id, &localhost, &tmp_email);
 remove(cam_tmp_email);
}

//======================================================
// Funcao para enviar um email                          
void msg_queue(char *email_conteudo, char *remetente, char *destinatario, char *add_to_header){
 FILE *arq_email, *local_filac, *local_filal;
 char local_id[TAM_id_fila];
 char local_cam_filac[200];
 char local_cam_filal[200];
 char buffer_local[TAM_buffer];
 char temp_data[32];
 unsigned int cont1=0;
 
 if(!*destinatario)return;
 if(!(arq_email=fopen(email_conteudo, "rb")))return;
 
 if(!id_pid[0]){
  time(&tempo);
  selor_superstrncpy(local_id, TAM_id_fila, itoa(tempo, 0), "_", fpid, NULL);
 }else strcpy(local_id, id_pid);
 selor_strncat(local_id, itoa(msg_cont++, 0), TAM_id_fila);
 selor_superstrncpy(local_cam_filac, sizeof(local_cam_filac), DEF_queue_path, "/c", local_id, NULL);
 selor_superstrncpy(local_cam_filal, sizeof(local_cam_filal), DEF_queue_path, "/_l", local_id, NULL);
 if(!(local_filac=fopen(local_cam_filac, "wb")))return;
 if(!(local_filal=fopen(local_cam_filal, "wb")))return;
 
 // -- Gravando Arquivo Lista --
 cont1=selor_superstrncpy(buffer_local, TAM_buffer, "<", remetente, "> 0 0\n[  ", destinatario, "]\n", NULL);
 fwrite(&buffer_local, 1, cont1, local_filal);
 fclose(local_filal);
 
 // -- Gravando Arquivo Conteudo --
 cont1=selor_superstrncpy(buffer_local, TAM_buffer, "Return-Path: <", remetente, ">\nReceived: from localhost (localhost[127.0.0.1])\n\tby "
                         ,DEF_server_name, " SMTP SELOR ", VERSAO, " id ", local_id, "\n", "\tfor <", destinatario, ">; " , NULL);
 fwrite(&buffer_local, 1, cont1, local_filac);
 rfc_date(temp_data, tempo);      // Timezone ja  embutido por esta funcao
 cont1=selor_superstrncpy(buffer_local, TAM_buffer, temp_data, "\nMessage-ID: <", local_id, "@", DEF_server_name, ">\n",
                         "Date: ", temp_data, "\n", NULL);
 fwrite(&buffer_local, 1, cont1, local_filac);
 if(add_to_header){
  cont1=special_cp(buffer_local, add_to_header, TAM_buffer-5);
  if(buffer_local[cont1-1] != '\n')buffer_local[cont1++]='\n';
  fwrite(buffer_local, 1, cont1, local_filac);
 }
 while(!feof(arq_email)){
  cont1=fread(buffer_local, 1, TAM_buffer, arq_email);
  fwrite(buffer_local, 1, cont1, local_filac);
 }
 fclose(local_filac);
 fclose(arq_email);
 selor_superstrncpy(local_cam_filac, sizeof(local_cam_filac), DEF_queue_path, "/l", local_id, NULL);
 rename(local_cam_filal, local_cam_filac);
}



//======================================================
// Funcao para tratar avisos de entrega e erro de um arq
// de fila                                              
// Retornos:                                            
// -1 = Erro ao criar arquivos temporarios              
// 0 = Arquivo processo com sucesso                     
// 1 = Arquivo processo com sucesso e FINALIZADO        
// 2 = Email recusado pelo destinatario                 
char tratar_erros(char *id, FILE *arq_lid, char *cam_cid){
 FILE *arq_tmp=NULL, *arq_erro=NULL; //, *arq_ae;
 struct cpl_struct arq_handler;
 char cam_tmp_email[TAM_cam_email];
 char arq_erro_criado=0;
 char ip[16];
 char local=1;
 char finalizado=1;             // --> Por padrao indica que foi finalizado
 int b_lidos=0;
 unsigned int cont1=0;
 
 
 if(!from.email[0]){
  // Remetente NULO. Nao ha o que reportar
  while(to[cont1].email[0]){
   if(to[cont1].status == STATUS_ERRO_TEMP){
    to[cont1].status=STATUS_PENDENTE;
    finalizado=0;
   }else if(to[cont1].status == STATUS_PENDENTE)finalizado=0;
   cont1++;
  }
  return finalizado;
 }
 ip[0]=0;       //Zerada aqui para o Compilador nao usar memset
 
 // -- Olhando lista de destinatarios --
 while(to[cont1].email[0]){
  if(to[cont1].status == STATUS_ERRO_TEMP){
   to[cont1].status=STATUS_PENDENTE;
   finalizado=0;
  }else if(to[cont1].status == STATUS_ERRO){
   // -- Tem erro para tratar
   if(!arq_erro_criado){
    // -- Criando arquivo de MAILER-DAEMON
    if(DEF_event_error_report[0]){
     R_CLOSE=R_DENY=0;
     smtp_dados_email(DEF_report_from, &from_tmp, NULL);
     exec_regras(DEF_event_error_report, NULL, &from_tmp, &from);
     if(R_CLOSE || R_DENY)return 1;
    }
    selor_superstrncpy(cam_tmp_email, TAM_buffer, DEF_temp_path, "/e", id, ".tmp", NULL);
    if(!(arq_erro=fopen(cam_tmp_email, "wb"))){
     // Erro criando arquivo temporario
     selor_superstrncpy(buffer_log, TAM_buffer_log, "(PID ", fpid, ") ERROR: Could not create the temporary file for error report [", linha, "]\n", NULL);
     log_msg(DEF_warning_log);
     return -1;
    }
    rfc_date(linha, tempo);      // Timezone ja  embutido por esta funcao
    b_lidos=selor_superstrncpy(buffer, TAM_buffer, "Return-Path: ", DEF_report_from, "\nReceived: from localhost (localhost[127.0.0.1])\n\tby ", DEF_server_name, " SMTP SELOR v", VERSAO, " id ", id, "\n\tfor <", from.email, ">; ", linha, "\n", NULL);
    fwrite(&buffer, 1, b_lidos, arq_erro);
    b_lidos=selor_superstrncpy(buffer, TAM_buffer, "Date: ", linha, "\nFrom: MAILER-DAEMON ", DEF_report_from, "\nTo: ", from.email, "\nSubject: ", DEF_subject_error_msg, "\n\n", NULL);
    fwrite(&buffer, 1, b_lidos, arq_erro);
    if(DEF_report_file[0]){
     if(!(arq_tmp=fopen(DEF_report_file, "rb"))){
      // Erro abrindo arquivo report
      selor_superstrncpy(buffer_log, TAM_buffer_log, "(PID ", fpid, ") WRN: Could not open file [", DEF_report_file, "]\n", NULL);
      log_msg(DEF_warning_log);
     }else {
      while(!feof(arq_tmp)){              // Copiando arquivo DEF_report_file para 
       b_lidos=fread(&buffer, 1, TAM_buffer, arq_tmp);
       if(b_lidos > 0)fwrite(&buffer, 1, b_lidos, arq_erro);
      }
      fwrite("\n\n", 1, 2, arq_erro);
      fclose(arq_tmp);
     }
    }
    arq_erro_criado=1;
   }
   // -- Gravando lista de erros
   strcpy(&to[cont1].resp[TAM_email_resp-4], "...");
   b_lidos=selor_superstrncpy(buffer, TAM_buffer, "Error: ", to[cont1].email, " ", to[cont1].resp, "\n", NULL);
   fwrite(&buffer, 1, b_lidos, arq_erro);
   to[cont1].status=STATUS_FINALIZADO;
  }else if(to[cont1].status == STATUS_PENDENTE)finalizado=0;
  cont1++;
 }
 if(arq_erro_criado){
  if((arq_tmp=fopen(cam_cid, "rb"))){
   // Colocando o header do email na mensagem de resposta
   if(local)fwrite("\n\n=-=-=-=-=-=-=-=-=-=-=//=-=-=-=-=-=-=-=-=-=-=\nMail HEADER:\n\n", 1, 61, arq_erro);
   else selor_send(socket_remoto, "\r\n\r\n=-=-=-=-=-=-=-=-=-=-=//=-=-=-=-=-=-=-=-=-=-=\r\nMail HEADER:\r\n\r\n", 66, 0);
   arq_handler.dest=linha;
   arq_handler.tambuffer=arq_handler.tamusado=TAM_buffer;
   while(!feof(arq_tmp)){
    b_lidos=fread(buffer, 1, TAM_buffer, arq_tmp);
    buffer[b_lidos]=0;
    arq_handler.orig=buffer;
    while(!cpl(&arq_handler)){
     if(!linha[0])goto Fim_header;
     cont1=arq_handler.dest-linha;   // Pegando o tamanho da linha
     arq_handler.dest=linha;
     if(local){
      linha[cont1++]='\n';
      fwrite(&linha, 1, cont1, arq_erro);
     }else {
      linha[cont1++]='\r';
      linha[cont1++]='\n';
      selor_send(socket_remoto, linha, cont1, 0);
     }
    }
   }
   Fim_header:
   fclose(arq_tmp);
  }
  // Enviando o email de erro
  fclose(arq_erro);
  smtp_dados_email(DEF_report_from, &localhost, NULL);
  sendmail(cam_tmp_email, id, &localhost, &from);
  remove(cam_tmp_email);
 }
 
 return finalizado;
}

//======================================================
// Funcao para processar um arquivo de fila             
// Retornos:                                            
// 0 = Arquivo processado com sucesso                   
// 1 = Arquivo nao existe                               
// 2 = Erro interno                                     
// 3 = Arquivo em uso por outro processo                
// 4 = Load Average                                     
char proc_id(char *id){
 FILE *arq_lid, *arq_cid;
 time_t tarpit_time=0;
 time_t atual_tempo=0;
 struct sockaddr_in rede_local;
 struct cpl_struct arq_handler;
 char arq_lid_sessao=0;
 char ip[16];
 char ip_source[16];
 char loop_tarpit=0;
 char hadd_ativo=0;
 char *ptr_aux1;
 char *ptr_aux2;
 char *ptr_aux3;
 int b_lidos=0;
 int num_tentativas=0;
 int num_report_erros=0;
 int email_index=0;
 int arq_lock=0;
 int ret_op=0;
 int tam_hadd=0;
 unsigned short porta=0;
 unsigned int ipn=0;
 unsigned int cont1=0;
 unsigned int cont2=0;
 unsigned int cont3=0;
 unsigned int cont4=0;
 unsigned int rcpts_aceitos=0;
 unsigned int resp_mx_query=0;
 unsigned int num_mxzes=0;
 unsigned int tarpit_control=0;
 unsigned int tam_email=0;
 
 
 if(getloadavg(&load_avg, 1) && load_avg >= (double)DEF_lavg_queue)return 4;
 
 etapa_global=3.0;
 tls_ativo=0;
 num_ops=0;
 ip[0]=0;        //Zerada aqui para o Compilador nao usar memset
 log_resumo_tipo[2]=0;
 // -- Capturando/Ignorando sinais --
 signal(SIGPIPE, SIG_IGN);
 selor_superstrncpy(cam_filac, sizeof(cam_filac)-1, DEF_queue_path, "/c", id, NULL);
 selor_superstrncpy(cam_filal, sizeof(cam_filal)-1, DEF_queue_path, "/l", id, NULL);
 arq_lid=fopen(cam_filal, "rb+");
 arq_lock=open(cam_filal, O_RDWR);
 if(arq_lock < 0 || !arq_lid)return 1;
 if(lockf(arq_lock, F_TLOCK, 0) !=0){               /* Mantendo travado apenas o arq da lista */
  close(arq_lock);
  return 3;
 }
 selor_superstrncpy(buffer_log, TAM_buffer_log, "(PID ", fpid, ") Proc. ID ", id, "\n", NULL);
 log_msg(DEF_central_log);
 if(!(arq_cid=fopen(cam_filac, "rb"))){
  // Erro abrindo email
  selor_superstrncpy(buffer_log, TAM_buffer_log, "(PID ", fpid, ") ERROR: Could not open file [", cam_filac, "\n", NULL);
  log_msg(DEF_warning_log);
  fclose(arq_lid);
  close(arq_lock);
  return 1;
 }
 fseek(arq_cid, 0, 2); //Posicionando no final do arquivo para pegar o tamanho dele sem precisar dar um stat
 tam_email=ftell(arq_cid);
 strcpy(log_resumo_size, itoa(tam_email, 0));
 if(strcmp(id, id_fila)){                       // Para ver se a lista de rcpts da fila ja esta carregada
  // -- Carregando arquivo de lista de rcpts --
  etapa_global=3.1;
  arq_handler.dest=linha;
  arq_handler.tambuffer=arq_handler.tamusado=TAM_buffer;
  num_rcpt=-1;
  while(!feof(arq_lid)){
   b_lidos=fread(buffer, 1, TAM_buffer, arq_lid);
   buffer[b_lidos]=0;
   arq_handler.orig=buffer;
   while(!cpl(&arq_handler)){
    arq_handler.dest=linha;
    if(linha[0] == '['){
     num_rcpt++;
     to[num_rcpt].status=linha[1];
     if(to[num_rcpt].status == '*')to[num_rcpt].status='@';
     smtp_dados_email(&linha[2], &to[num_rcpt], NULL);
     to[num_rcpt].local=cache_mod_dom_local(to[num_rcpt].dominio, adomain, &cont2);
     to[num_rcpt].dest[0]=0;
     to[num_rcpt].auth[0]=0;
     to[num_rcpt].resp[0]=0;
     to[num_rcpt].so_mark=0;
     arq_lid_sessao=1;
    }else if(linha[0] == '<'){
     ptr_aux1=strchr(linha, '>');
     if(ptr_aux1){
      // Lendo numero de tentativas
      ptr_aux1++;
      ptr_aux1=pular_carac(ptr_aux1, ' ');
      num_tentativas=atoi(ptr_aux1);
      ptr_aux1=strchr(ptr_aux1, ' ');
      if(ptr_aux1){
       ptr_aux1=pular_carac(ptr_aux1, ' ');
       num_report_erros=atoi(ptr_aux1);
      }
     }
     smtp_dados_email(linha, &from, NULL);
     if(from.dominio)from.local=cache_mod_dom_local(from.dominio, adomain, &cont2);
     arq_lid_sessao=0;
    }else if(arq_lid_sessao){
     // Ler detalhes do destinatario
     ptr_aux1=linha;
     if(!strncmp(ptr_aux1, "Resp:", 5)){
      ptr_aux1+=5;
      ptr_aux1=pular_carac(ptr_aux1, ' ');
      selor_strncpy(to[num_rcpt].resp, ptr_aux1, TAM_email_resp);
     }else if(!strncmp(ptr_aux1, "Dest:", 5)){
      ptr_aux1+=5;
      ptr_aux1=pular_carac(ptr_aux1, ' ');
      selor_strncpy(to[num_rcpt].dest, ptr_aux1, TAM_email_dest);
     }else if(!strncmp(ptr_aux1, "Mx_auth:", 8)){
      ptr_aux1+=8;
      ptr_aux1=pular_carac(ptr_aux1, ' ');
      selor_strncpy(to[num_rcpt].auth, ptr_aux1, TAM_email_auth);
     }else if(!strncmp(ptr_aux1, "So_mark:", 8)){
      ptr_aux1+=8;
      ptr_aux1=pular_carac(ptr_aux1, ' ');
      to[num_rcpt].so_mark=atoi(ptr_aux1);
     }else if(!strncmp(ptr_aux1, "Mx_source:", 10)){
      ptr_aux1+=10;
      ptr_aux1=pular_carac(ptr_aux1, ' ');
      selor_strncpy(to[num_rcpt].mx_source, ptr_aux1, NS_MAXDNAME);
     }
    }else {
     // Ler detalhes globais
     ptr_aux1=linha;
     if(!strncmp(ptr_aux1, "Def:", 4)){
      hadd_ativo=0;
      ptr_aux1=&linha[4];
      if(!(ptr_aux1=strchr(ptr_aux1, ' ')))continue;
      *ptr_aux1++=0;
      ptr_aux1=pular_carac(ptr_aux1, ' ');
      if(*ptr_aux1 == '"'){
       ptr_aux1++;
       if((ptr_aux2=strchr(ptr_aux1, '"')))*ptr_aux2=0;
      }
      if((ret_op=def_parametro(&linha[4], ptr_aux1)) >=0)def_ops[num_ops++]=ret_op;
     }else if(!strncmp(ptr_aux1, "Hadd:", 5)){
      hadd_ativo=1;
      ptr_aux1+=5;
      ptr_aux1=pular_carac(ptr_aux1, ' ');
      tam_hadd=selor_superstrncpy(R_HADD, TAM_hadd, ptr_aux1, "\r\n", NULL);
     }else if(hadd_ativo){
      tam_hadd+=selor_superstrncpy(&R_HADD[tam_hadd], TAM_hadd-tam_hadd, ptr_aux1, "\r\n", NULL);
     }
    }
   }
  }
  num_rcpt++;
 }else num_rcpt=num_rcpts;
 
 
 // -------------------------------------------------
 // Criando mapa de MXzes para seus respectivos rcpts
 //memset(mx_map, 0, sizeof(struct str_mx_map) * DEF_max_rcpts);
 mx_map[0].ipn=Localhost;                      // localhost primeiro
 mx_map[0].finalizado=0;
 mx_map[0].so_mark=0;
 mx_map[0].mx_source[0]=0;
 mx_map[0].ini=mx_map[0].ult=-1;
 mx_map[1].ipn=0;
 num_mxzes=1;
 temp=0;
 etapa_global=3.2;
 for(cont1=0; cont1<num_rcpt; cont1++){
  if(to[cont1].status != '@' && to[cont1].status != '!'){        // -> Apenas para rcpts nao processados
   temp=1;
   if(to[cont1].local == -1){
    // Dominio indefinido. Vamos descobrir se  local ou nao
    if(to[cont1].dominio)to[cont1].local=cache_mod_dom_local(to[cont1].dominio, adomain, &cont2);
    else to[cont1].local=1; // Dominio NULO, provavel usuario local
   }
   if(to[cont1].local == 1){
    // Dominio Local
    Local:
    if(mx_map[0].ult < 0){
     mx_index[cont1]=-1;
     mx_map[0].ini=mx_map[0].ult=cont1; // cont1 aponta para a posicao do RCPT
    }else {
     mx_index[mx_map[0].ult]=cont1;     // Dominio Local
     mx_map[0].ult=cont1;               // cont1 aponta para a posicao do RCPT
    }
    mx_index[cont1]=-1;                 // -1 indica fim de lista para a atual thread de MX
   }else {
    // Dominio externo
    porta=PORTA_padrao;                // Apontando para a porta de entrega padrao SMTP (25)
    ptr_aux1=NULL;
    if(to[cont1].dest[0]){
     // Mx_dest! usado
     ipn=0;
     ptr_aux1=to[cont1].dest;
     if((ptr_aux2=strchr(ptr_aux1, ' '))){    // Procurando a indicacao de uma porta diferente da PORTA_padrao
      *ptr_aux2++=0;      // Isolando endereco do host para 'resolver_dom' nao ter problemas com o num. da porta
      ptr_aux2=pular_carac(ptr_aux2, ' ');
      porta=atoi(ptr_aux2);
     }
    }
    if(!ptr_aux1 || !*ptr_aux1)ptr_aux1=to[cont1].dominio;
    if(ptr_aux1 != to[cont1].dest){
     if(!dns_query(ptr_aux1, T_MX, &resp_mx_query)){
      // Procurando MX de menor preferencia
      cont4=0xFFFFFFFF;
      for(cont2=0; cont2<resp_mx_query; cont2++){
       if(cont4 > dns_resp[cont2].pref){
        cont4=dns_resp[cont2].pref;
        cont3=cont2;
       }
      }
      ipn=resolver_dom(dns_resp[cont3].nome, ip, 0);
     }else ipn=0;           // Para forcar resolver o dominio
    }
    if(!ipn && !(ipn=resolver_dom(ptr_aux1, ip, 0))){  // Resolvendo o campo A do dominio para tentar entregar o email
     // Dominio inexistente. Marcando RCPT com mensagem de errro
     to[cont1].status=STATUS_ERRO;
     selor_superstrncpy(to[cont1].resp, TAM_email_resp, "[", DEF_server_name, "] -> ", DEF_msg_cannot_resolve, NULL);
     continue; 
    }
    if(ip_local(ipn)){      // Testando se o MX nao aponta para o proprio servidor
     selor_superstrncpy(buffer_log, TAM_buffer_log, "(PID ", fpid, ") WRN: The domain ", to[cont1].dominio, " returns a local IP, but it is not in local list\n", NULL);
     log_msg(DEF_warning_log);
     to[cont1].local=1;
     goto Local;
    }
    for(cont2=0; cont2<num_mxzes; cont2++){
     if(mx_map[cont2].ipn == ipn && mx_map[cont2].porta == porta &&
        mx_map[cont2].so_mark == to[cont1].so_mark &&
        !cmp_str_vars(mx_map[cont2].mx_source, to[cont1].mx_source) &&
        !cmp_str_vars(mx_map[cont2].auth, to[cont1].auth))break;
    }
    if(cont2 == num_mxzes){
     // Nenhum MX localizado
     if(to[cont1].dest[0])mx_map[num_mxzes].forcado=1;
     mx_map[num_mxzes].finalizado=0;
     mx_map[num_mxzes].ipn=ipn;
     mx_map[num_mxzes].porta=porta;
     mx_map[num_mxzes].so_mark=to[cont1].so_mark;
     selor_strncpy(mx_map[num_mxzes].mx_source, to[cont1].mx_source, NS_MAXDNAME);
     selor_strncpy(mx_map[num_mxzes].auth, to[cont1].auth, TAM_email_auth);
     strcpy(mx_map[num_mxzes].ip, ip);
     mx_map[num_mxzes].ini=mx_map[num_mxzes].ult=cont1;    // cont1 aponta para a posicao do RCPT
     num_mxzes++;
     mx_map[num_mxzes].ipn=0;
    }else {
     mx_index[mx_map[cont2].ult]=cont1;
     mx_map[cont2].ult=cont1;  // cont1 aponta para a posicao do RCPT
    }
    mx_index[cont1]=-1;        // -1 indica fim de lista para a atual thread de MX
   }
  }
 }
 
 
 // -------------------------
 // Entregando Emails        
 // -------------------------
 etapa_global=3.3;
 if(from.local)log_resumo_tipo[0]='L';            // FROM -> Local  
 else if(from.dominio)log_resumo_tipo[0]='R';     // FROM -> Remoto 
 else log_resumo_tipo[0]='A';                     // FROM -> Anonimo
 num_tentativas++;
 do{
  cont1=0;
  loop_tarpit=0;
  while(mx_map[cont1].ipn){
   if(mx_map[cont1].finalizado){
    cont1++;
    continue;
   }
   mx_map[cont1].finalizado=1;
   email_index=mx_map[cont1].ini;
   if(!cont1){
    // -------------------
    // --> Email LOCAL <--
    etapa_global=3.4;
    log_resumo_tipo[1]='L';
    while(email_index >= 0){             // Varre toda lista de emails locais
     //if((mod_test_quota[0](to[email_index].email, mod_user_info, MOD_user_ncampos)) == 1){
     cont3=0;
     while(!(ret_val=mod_test_quota[modseq_mail[cont3]](to[email_index].email, mod_user_info, MOD_user_ncampos)) && (modseq_mail[++cont3] != 0xFF));
     if(ret_val == 2 || !ret_val){
      // Quota excedida ou Conta nao existe
      if (!ret_val)ptr_aux2=DEF_msg_no_mailbox;
      else ptr_aux2=DEF_msg_full_mailbox;
      resp_rcpt(email_index, "127.0.0.1", ptr_aux2);
      selor_superstrncpy(buffer_log, TAM_buffer_log, "(", id, ") <- 127.0.0.1 ", to[email_index].email, " ", ptr_aux2, "\n", NULL);
      log_msg(DEF_central_log);
     }else {
      if(dlocal(to[email_index].dest, cam_filac, id, &from, &to[email_index]))resp_rcpt(email_index, "127.0.0.1", DEF_msg_mailbox_unavaibble);   // -> Erro na entrega. Tentar depois
      else {
       resp_rcpt(email_index, ip, DEF_msg_ok);   // -> Email entregue
       selor_superstrncpy(buffer_log, TAM_buffer_log, log_resumo_tipo, " ID:", id, " from:<", from.email, "> to:<", to[email_index].email, "> size:", log_resumo_size, " transf:", log_resumo_size, "\n", NULL);
       log_msg(DEF_sumary_log);
      }
     }
     email_index=mx_index[email_index];  // Apontando para proximo email da lista
    }
   }else {
    // ---------------------
    // --> Email EXTERNO <--
    etapa_global=3.5;
    log_resumo_tipo[1]='R';
    strcpy(ip, mx_map[cont1].ip);
    ipn=mx_map[cont1].ipn;
    dns_num_mxzes=0;     // Zerando a quantidade de MXzes encontrados              
    dns_num_mx=0;        // Zerando o indice do MX para ser usado por prox_mx      
    dns_num_ip=0;        // Zerando o indice do IP do MX para ser usado por prox_mx
    do{
     // -- Tentando conectar-se a um MX --
     selor_superstrncpy(buffer_log, TAM_buffer_log, "(", id, ") Trying server ", ip, "\n", NULL);
     log_msg(DEF_central_log);
     if((socket_remoto=socket(AF_INET, SOCK_STREAM, 0)) == -1){
      // Erro no socket
      selor_superstrncpy(buffer_log, TAM_buffer_log, "(PID ", fpid, ") ERROR: ", strerror(errno), "\n", NULL);
      log_msg(DEF_warning_log);
      close(arq_lock);
      return 2;
     }
     if(mx_map[cont1].so_mark)setsockopt(socket_remoto, SOL_SOCKET, SO_MARK, &mx_map[cont1].so_mark, sizeof(int));
     if(mx_map[cont1].mx_source[0]){
      // Alterando IP de origem
      ip_source[0]=0;
      rede_local.sin_family=AF_INET;
      rede_local.sin_port=0;
      if((rede_local.sin_addr.s_addr=resolver_dom(mx_map[cont1].mx_source, ip_source, 0))){
       memset(&rede_local.sin_zero,0,8);
       if(bind(socket_remoto,(struct sockaddr *)&rede_local,sizeof(rede_local))){
        selor_superstrncpy(buffer_log, TAM_buffer_log, "(PID ", fpid, ") ERROR: Mx_source ", ip_source, " -> ", strerror(errno), "\n", NULL);
        log_msg(DEF_warning_log);
        mx_map[cont1].mx_source[0]=0;
       }
      }else {
       selor_superstrncpy(buffer_log, TAM_buffer_log, "(PID ", fpid, ") ERROR: Mx_source ", mx_map[cont1].mx_source, " -> Can't resolve\n", NULL);
       log_msg(DEF_warning_log);
       mx_map[cont1].mx_source[0]=0;
      }
     }
     rede_remota.sin_family=AF_INET;
     rede_remota.sin_port=htons(mx_map[cont1].porta);
     rede_remota.sin_addr.s_addr=ipn;
     memset(&rede_remota.sin_zero, 0, 8);
     alarm(DEF_connect_timeout);
     cont2=connect(socket_remoto, (struct sockaddr *)&rede_remota, sizeof(struct sockaddr));
     alarm(0);
     if(cont2){
      // Erro ao conectar. Vamos tentar outro MX
      if(mx_map[cont1].forcado || (!dns_num_mx && !dns_num_ip && achar_mx(to[email_index].dominio, &num_mxzes)) || (!(ipn=prox_mx(ip, DEF_mx_alias_ip)))){
       cont2=-1;  // Nao conectado. Dominio nao existe
       break;     // Sair da procura de MXzes         
      }
     }
    }while(cont2);
    if(!cont2){
     // -- Conectado, entregando email --
     tls_ativo=0;
     if(!mx_map[cont1].mx_source[0])selor_superstrncpy(buffer_log, TAM_buffer_log, "(", id, ") Connected - ", ip, ":", itoa(mx_map[cont1].porta, 0), "\n", NULL);
     else selor_superstrncpy(buffer_log, TAM_buffer_log, "(", id, ") Connected - ", ip, ":", itoa(mx_map[cont1].porta, 0), " [src:", mx_map[cont1].mx_source, "]\n", NULL);
     log_msg(DEF_central_log);
     b_lidos=selor_smtp_recv(socket_remoto, buffer, TAM_buffer, 0);
     if(b_lidos < 1){
      // Conexao fechada
      Cnx_fechada:
      selor_superstrncpy(buffer_log, TAM_buffer_log, "(", id, ") - ", ip, " Connection timeout/broken\n", NULL);
      log_msg(DEF_central_log);
      cont1++;                               // Proximo MX              
      continue;
     }
     if(buffer[0] != '2'){
      // Servidor nao disponivel
      Marcar_erros:
      str_arruma_crlf(buffer);
      selor_superstrncpy(buffer_log, TAM_buffer_log, "(", id, ") <- ", ip, " ", buffer, "\n", NULL);
      log_msg(DEF_central_log);
      resp_rcpts(email_index, ip, buffer, id);   // Marcando os rcpts do MX 
      cont1++;                                   // Proximo MX              
      gravar_lista(num_rcpt, arq_lid, cam_filal, num_tentativas, num_report_erros);
      continue;
     }
     temp=cnx_smtp_init(socket_remoto, id, ip, mx_map[cont1].auth, from.email, 1);
     if(temp & 0x1)goto Cnx_fechada;
     if(temp & 0x2)goto Marcar_erros;
     rcpts_aceitos=0;
     tarpit_control=0;
     etapa_global=3.6;
     while(email_index >= 0){                // Varre toda a lista do atual MX
     if(tarpit_control < DEF_tarpit_limit){
       if(to[email_index].status == STATUS_PENDENTE){         // Colocado para nao processar novamente erros 4xx em caso de tarpit
        b_lidos=selor_superstrncpy(buffer, TAM_buffer, "rcpt to: <", to[email_index].email, ">\r\n", NULL);
        selor_send(socket_remoto, buffer, b_lidos, 0);
        selor_superstrncpy(buffer_log, TAM_buffer_log, "(", id, ") -> ", ip, " ", buffer, NULL);
        log_msg(DEF_central_log);
        tarpit_control++;
        b_lidos=selor_smtp_recv(socket_remoto, buffer, TAM_buffer, 0);
        if(b_lidos < 1)goto Cnx_fechada;
        if(DEF_event_dl_rcpt[0]){            // --> Event.dl_rcpt
         // Gravando msg de erro na variavel: %T da RBL
         selor_strncpy(R_SERVER_RESP, buffer, sizeof(R_SERVER_RESP));
         ptr_aux2=R_SERVER_RESP;
         while(*ptr_aux2){
          if(*ptr_aux2 == '\'' || *ptr_aux2 == '"')*ptr_aux2=' ';
          ptr_aux2++;
         }
         exec_regras(DEF_event_dl_rcpt, NULL, &from, &to[email_index]);
        }
        if(buffer[0] != '2'){
         str_arruma_crlf(buffer);
         selor_superstrncpy(buffer_log, TAM_buffer_log, "(", id, ") <- ", ip, " ", buffer, "\n", NULL);
         log_msg(DEF_central_log);
         resp_rcpt(email_index, ip, buffer);   // Destinatario recusado. Marcando-o
        }else rcpts_aceitos++;
       }
      }else {
       to[email_index].status=STATUS_TARPIT;
       loop_tarpit=1;
       mx_map[cont1].finalizado=0;
       time(&tarpit_time);
      }
      email_index=mx_index[email_index];     // Apontando para proximo email da lista
     }
     if(rcpts_aceitos){
      // -- Enviando conteudo do email --
      etapa_global=3.7;
      selor_send(socket_remoto, "data\r\n", 6, 0);
      selor_superstrncpy(buffer_log, TAM_buffer_log, "(", id, ") -> ", ip, " data\n", NULL);
      log_msg(DEF_central_log);
      b_lidos=selor_smtp_recv(socket_remoto, buffer, TAM_buffer, 0);
      if(b_lidos < 1)goto Cnx_fechada;
      if(buffer[0] != '3' && buffer[1] != '5' && buffer[2] != '4'){
       email_index=mx_map[cont1].ini;
       goto Marcar_erros;  // Data recusado. Vamos marcar os rcpts
      }
      if(R_HADD[0]){
       // Enviando Informacao extra do Hadd!
       b_lidos=strlen(R_HADD);
       if(R_HADD[b_lidos-2] != 13){
        // Corrindo CRLF
        R_HADD[b_lidos-1]=13;
        R_HADD[b_lidos++]=10;
        R_HADD[b_lidos]=0;
       }
       selor_send(socket_remoto, R_HADD, b_lidos, 0);
      }
      
      // Procurando a posicao no arquivo depois do primeiro Return-PATH
      fseek(arq_cid, 0, 0);
      b_lidos=fread(buffer, 1, 512, arq_cid);
      if(b_lidos > 0){
       buffer[b_lidos]=0;
       if(!cmp_str(buffer, "Return-Path:", 12)){
        ptr_aux3=strchr(buffer, 10);
        if(ptr_aux3){
         ptr_aux3++;
         fseek(arq_cid, ptr_aux3-buffer, 0);
        }else fseek(arq_cid, 0, 0);
       }else fseek(arq_cid, 0, 0);
      }else fseek(arq_cid, 0, 0);
      
      if(data_env_arq(arq_cid, 1))goto Cnx_fechada;
      b_lidos=selor_smtp_recv(socket_remoto, buffer, TAM_buffer, 0);
      if(b_lidos < 1)goto Cnx_fechada;
      if(buffer[0] != '2'){
       email_index=mx_map[cont1].ini;
       goto Marcar_erros;
      }
      resp_rcpts(mx_map[cont1].ini, ip, buffer, id);   // Marcando os rcpts com mensagem de sucesso
      selor_superstrncpy(buffer_log, TAM_buffer_log, "(", id, ") <- ", ip, " ", buffer, "\n", NULL);
      log_msg(DEF_central_log);
     }
     gravar_lista(num_rcpt, arq_lid, cam_filal, num_tentativas, num_report_erros);
     selor_send(socket_remoto, "quit\r\n", 6, 0);
     b_lidos=selor_smtp_recv(socket_remoto, linha, TAM_buffer, 0);
     close(socket_remoto);
    }
   }
   cont1++;
  }
  if(loop_tarpit && DEF_tarpit_sleep){
   time(&atual_tempo);
   if(atual_tempo-tarpit_time < DEF_tarpit_sleep)sleep(DEF_tarpit_sleep-(atual_tempo-tarpit_time));
  }
 }while(loop_tarpit);
 if(num_tentativas >= DEF_max_attempts){
  for(cont1=0; cont1<num_rcpt; cont1++){
   if(to[cont1].status == STATUS_PENDENTE){
    to[cont1].status=STATUS_ERRO;
    if(!to[cont1].resp[0])selor_strncpy(to[cont1].resp, DEF_msg_cannot_connect, TAM_email_resp);
   }else if(to[cont1].status == STATUS_ERRO_TEMP)to[cont1].status=STATUS_ERRO;
  }
 }
 tls_ativo=0;
 
 etapa_global=3.8;
 if((b_lidos=tratar_erros(id, arq_lid, cam_filac)) == 2)num_report_erros++;
 if(num_report_erros >= DEF_max_error_report){
  postmaster_report(id, arq_lid);
  b_lidos=1;             // Para finalizar o ID
 }else gravar_lista(num_rcpt, arq_lid, cam_filal, num_tentativas, num_report_erros);
 fclose(arq_cid);
 fclose(arq_lid);
 lockf(arq_lock, F_ULOCK, 0);
 close(arq_lock);
 
 // Processando qend_process
 if(DEF_event_qend_process[0])exec_regras(DEF_event_qend_process, NULL, &from, atual_rcpt);
 close(arq_lock);
 if(b_lidos == 1){
  selor_superstrncpy(buffer_log, TAM_buffer_log, "(", id, ") - Finished\n", NULL);
  log_msg(DEF_central_log);
  remove(cam_filac);
  remove(cam_filal);
 }
 return 0;
}



